'異方性3D熱伝導解析V105の標準ﾓｼﾞｭｰﾙ      ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module AnIsoTCom
    'アイソパラメトリック六面体要素使用
    '自動メッシュ生成は「ＦＥＭ立体データ」ソフトによる
    '要素の節点番号の入力は、指定順序の反時計回り
    '初期温度の一括設定機能有り
    '等温線図はカラーフィル有無の選択可
    '各作画画面において、作画パラメーターの調整やマウスドラッグによる部分拡大が可能

    'V96での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V97での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V98での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V99での改良点・・・構造内の面間の熱放射グループを追加（六面体要素のみ）、
    '                   定常解析では熱放射境界をエラー表示、
    '                   内部面境界の設定削除を機能追加
    'V100での改良点・・・内部面境界の設定削除でデータ変更有りに変更、
    '                    構造図画面で「隠面処理なし」の選択を廃止、
    '                    構造図画面で条件設定部の表示を２区分から３区分に改善
    'V101での改良点・・・構造図画面で条件設定部のテキスト間違いを訂正
    'V102での改良点・・・四面体要素の廃止
    'V103での改良点・・・解析データのデータチェックに立体要素共通面の節点を追加
    'V104での改良点・・・実行形式は.exeと.exe.configを同一フォルダに配置
    'V105での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Pai As Single = 3.141593  'π0

    Public CalcRes$         '計算結果表示データ
    Public Cr$              'CR+LFのコード 
    Public DirOpen$         '立ち上げ時のディレクトリー名
    Public Fadd$            '連結するファイル名
    Public Fgraph$                  'ｸﾞﾗﾌﾃﾞｰﾀの出力ﾌｧｲﾙ名
    Public Fm$              '入力データのファイル名
    Public Fy$              '計算結果のファイル名
    Public Title$           'タイトル

    Public Timst As Date          '計算開始時刻

    Public Idcycle As Integer       '出力サイクル間隔
    Public InitCond As Integer      'ﾊﾞｯﾁ処理で計算条件設定の有無（0:なし、1:有り）
    Public InitData As Integer      '初期値データの有無（0:なし、1:有り）
    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力、2:AnIsoFEMﾌｧｲﾙ入力）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '共通辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public Kcycle As Integer        '作画指定のサイクル番号
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KdispENum As Integer     '要素番号の表示有無（0:なし、1:有り）
    Public KdispMat As Integer      '作画の材料の指定番号（0は全材料）
    Public KdispPNum As Integer     '節点番号の表示有無（0:なし、1:有り）
    Public KdispSNum As Integer     '面番号の表示有無（0:なし、1:有り）
    Public KDrawDatMod As Integer   '構造図でのﾃﾞｰﾀ修正区分（0:個別、1:一括）
    Public KdumyStart As Integer    'ダミー化開始ステップ番号
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public Kfinish As Integer       'ﾊﾞｯﾁ処理１回の終了区分（0:未終了、1:終了）
    Public KindBatch As Integer     'ﾊﾞｯﾁ処理の有無（0:なし、1:有り）
    Public KindBoundS As Integer    '一括設定の境界種別（0:温度固定、1:熱流束、2:熱伝達、3:熱放射）
    Public KindElem As Integer      '要素種類（0:六面体20節点、1:四面体10節点）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindPost As Integer      '後処理の種類（1:等温線図、2:等温線図ｽﾗｲﾄﾞｼｮｰ、3:主要値表示、
    '                                4:非定常のｸﾞﾗﾌﾃﾞｰﾀ）
    Public KindPref As Integer      '優先選択（0:計算時間、1:ﾒﾓﾘｰ使用量）
    Public KindSolver As Integer    'ｿﾙﾊﾞｰの種類（0:ｽｶｲﾗｲﾝ法、1:反復法）
    Public KmatDumy As Integer      'ダミー化の材料番号
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:材料番号、2:温度固定節点、3:温度固定境界面、
    '                                4:熱流束、5:熱伝達、6:熱放射、7:内部発熱）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KmouseSurf As Integer    'ﾏｳｽ指定の面番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public KStage As Integer        'ﾃﾞｰﾀ入力の段階番号
    Public Kstep As Integer         '作画指定のステップ番号
    Public Ktime As Integer         '出力メッシュ番号
    Public Mprop As Integer         '材料データの成分数(=5)
    Public Mstat As Integer         '解析種別（０:定常、１:非定常）
    Public Nall As Integer          '変数の総数
    Public Namb, Nambb As Integer          '熱伝達境界の面の数
    Public Nbound As Integer        '温度固定の全節点数
    Public NboundP, NboundPb As Integer       '温度固定節点数
    Public NboundS, NboundSb As Integer       '温度固定面数
    Public Ncycle As Integer        '境界条件のサイクル数
    Public Ndel As Integer          '削除要素数
    Public Ndim As Integer          '１節点当りの変数の数(=1)
    Public NDisp As Integer         '等高線の分割数
    Public Nelem As Integer         '要素数
    Public Nflux, Nfluxb As Integer '熱流束境界の面の数
    Public Ngaus As Integer         'ガウス積分点数(=3)
    Public Nmat As Integer          '材料データ数
    Public NMdivMax As Integer      '材料の物性区分数最大値
    Public Npoint As Integer        '節点数
    Public NPgraph As Integer       'ｸﾞﾗﾌﾃﾞｰﾀの節点数
    Public Nqint, Nqintb As Integer '内部発熱の要素数
    Public Nrad, Nradb As Integer   '熱放射境界の面の数
    Public NradG, NradGb As Integer '熱放射グループの数
    Public NumRmax As Integer       '熱放射グループの最大面数
    Public NSdraw As Integer        '作画面の数（隠面処理時）
    Public Nstep As Integer         '境界条件のステップ数
    Public Ntime As Integer         '非定常の出力時間メッシュ数
    Public Nvar As Integer          '１要素当りの変数の総数（=20）

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public DtempMax As Single       '非定常の計算1回の温度変化許容値
    Public Dtime As Single          '非定常の出力の計算時間間隔
    Public DtimeInit As Single      '非定常の計算時間間隔の初期値
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public RateDumy As Single       'ダミー化の熱容量比率
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public SigmaG As Single         '熱放射グループのｽﾃﾌｧﾝ･ﾎﾞﾙﾂﾏﾝ定数（共通）
    Public TimeMax As Single        '非定常の打切り時間
    Public TimeNew As Single        '現在の計算時刻
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public UFileA(6) As String      '最近使用したﾃﾞｰﾀﾌｧｲﾙの保存配列

    Public KAppPoint() As Integer   'ﾃﾞｰﾀ設定の節点
    Public KAppSide(,) As Integer   'ﾃﾞｰﾀ設定の要素指定面
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KdrawElemOld() As Integer    '変更前の各要素の作画有無（0:なし、1:有り）
    Public KE3DCut(2) As Integer    '構造図の面切断の指定（座標軸番号と正負選択）
    Public KEInside(,) As Integer   '全体構造での要素各面の内部区分（0:外周、1:内部）
    Public Kelemq() As Integer      '内部発熱の要素番号
    Public KEorder() As Integer     '作画順の要素番号
    Public KinnerLine() As Integer  '内部辺の指定（0:なし、1:有り）
    Public Koption(3) As Integer    'オプション有無（1:バックアップ、
    Public KPgraph(20) As Integer   'ｸﾞﾗﾌ節点番号
    '                                2:要素入力画面の節点番号表示、3:節点入力画面の表示）
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public KSelem(,) As Integer     '各要素面の隠面指定（0:なし、1：有り、-1:視線外）
    Public KSorder() As Integer     '作画順の要素の面番号
    Public KsurfAmb(,) As Integer   '熱伝達境界指定の要素・面番号
    Public KsurfFlux(,) As Integer  '熱流束境界指定の要素・面番号
    Public KsurfPoint(8) As Integer '面の8節点の節点順位
    Public KsurfRad(,) As Integer   '熱放射境界指定の要素・面番号
    Public KsurfRest(,) As Integer  '固定温度境界指定の要素・面番号
    Public NMdiv() As Integer       '材料の物性区分数
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public Nrest() As Integer       '温度固定の全節点番号
    Public NrestP() As Integer      '温度固定の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号

    Public MemRad(,,) As Integer    '熱放射グループの面の要素番号・面番号
    Public NumRad() As Integer      '熱放射グループの面の数

    Public Alpha(,) As Single       '熱伝達係数
    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public BoundSVal(,) As Single   '境界一括設定の熱条件値
    Public CenterPos(3) As Single   '構造中心の座標
    Public Cspec(,) As Single       '材料の比熱
    Public Ds(20), Dt(20), Du(20) As Double  '形状関数のξ偏微分、η偏微分、ζ偏微分
    Public Ds2(20), Dt2(20), Du2(20) As Double  '形状関数2のξ偏微分、η偏微分、ζ偏微分

    Public Dx(20), Dy(20), Dz(20) As Double  '形状関数のx偏微分、y偏微分、z偏微分
    Public Dx2(20), Dy2(20), Dz2(20) As Double  '形状関数2のx偏微分、y偏微分、z偏微分

    Public EpsRad(,) As Single      '熱放射グループの面の放射率
    Public Pnorm(,) As Single       '単位面積当りの垂直熱流束
    Public PosGaus(3) As Single     '積分点の位置
    Public Qint(,) As Single        '内部発熱率（単位体積、単位時間当り）
    Public RadRate(,) As Single     '熱放射の係数(ｽﾃﾌｧﾝﾎﾞﾙﾂﾏﾝ定数×放射率×形状係数)
    Public RinvJ(3, 3) As Double    '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public RJmat(3, 3) As Double    'ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public RlamdaX(,) As Single     '材料のX方向熱伝導率
    Public RlamdaY(,) As Single     '材料のY方向熱伝導率
    Public RlamdaZ(,) As Single     '材料のZ方向熱伝導率
    Public Rou() As Single          '材料の密度
    Public Shape(20) As Double      '形状関数

    Public Shape2(20) As Double     '形状関数2
    Public TE(20) As Double         '要素節点の温度
    Public TempAmb(,) As Single     '熱伝達の外部温度
    Public TempFix(,) As Single     '全固定温度
    Public TempFixP(,) As Single    '節点の固定温度
    Public TempFixS(,) As Single    '面の固定温度
    Public TempInit() As Single     '初期温度
    Public TempMat(,) As Single     '材料の区分上限温度
    Public TempRad(,) As Single     '熱放射源温度
    Public TG(27) As Double         'ｶﾞｳｽ点の温度
    Public Theta1(), Theta2() As Single   '円弧の開始、終了角度（ﾗｼﾞｱﾝ、左回りに整理済）
    Public TimeStep() As Single     'ステップ時間
    Public TransMat(4, 4) As Single  'モデルの合成アフィン変換行列
    Public Tu() As Single           'FEM計算の解
    Public VE3DCut(2) As Single     '構造図の面切断の座標値と許容誤差
    Public WeightG(3) As Single     '積分点の重み
    Public Xdata(), Ydata(), Zdata() As Single    '節点座標
    Public Xdraw(8), Ydraw(8) As Single   '色塗り多角形の座標
    Public XE(20), YE(20), ZE(20) As Single   '要素計算の配列
    Public XE2(20), YE2(20), ZE2(20) As Single  '要素計算の配列2
    Public XG(27), YG(27), ZG(27) As Double     'ｶﾞｳｽ点の座標
    Public XG2(9), YG2(9), ZG2(9) As Double     'ｶﾞｳｽ点の座標2
    Public Xmodel(), Ymodel(), Zmodel() As Single 'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xsect(), Ysect(), Zsect() As Single  '部分断面の位置座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）

    Sub ADatDupCheck(ByRef nd As Integer)
        '熱伝達の重複ﾁｪｯｸ･修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Namb) As Integer

        nd = 0
        For i = 1 To Namb - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Namb
                    If KsurfAmb(i, 1) = KsurfAmb(ii, 1) Then
                        If KsurfAmb(i, 2) = KsurfAmb(ii, 2) Then
                            nd = nd + 1
                            kbuf(ii) = -1
                            For j = 1 To Nstep
                                Alpha(i, j) = Alpha(ii, j)
                                TempAmb(i, j) = TempAmb(ii, j)
                            Next j
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Namb, 2) As Integer
            Dim buf2(Namb, Nstep), buf3(Namb, Nstep) As Single
            For i = 1 To Namb
                For j = 1 To 2
                    nbuf1(i, j) = KsurfAmb(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = Alpha(i, j)
                    buf3(i, j) = TempAmb(i, j)
                Next j
            Next i
            nb = Namb - nd
            ReDim KsurfAmb(nb, 2), Alpha(nb, Nstep), TempAmb(nb, Nstep)
            nb = 0
            For i = 1 To Namb
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 2
                        KsurfAmb(nb, j) = nbuf1(i, j)
                    Next j
                    For j = 1 To Nstep
                        Alpha(nb, j) = buf2(i, j)
                        TempAmb(nb, j) = buf3(i, j)
                    Next j
                End If
            Next i
            Namb = nb
            Nambb = nb
        End If

    End Sub

    Sub AreaCalc(ByRef area As Single, ByVal x1 As Single,
    ByVal y1 As Single, ByVal x2 As Single, ByVal y2 As Single,
    ByVal xb As Single, ByVal yb As Single)
        '三角形の符号付き面積算出（最初の2点に対して左側は正、右側は負）
        'area･･･符号付き面積、三角形の2倍
        'x1,y1,x2,y2･･･2点の座標
        'xb,yb･･･対象点の座標
        area = x1 * y2 - y1 * x2 + x2 * yb - y2 * xb + xb * y1 - yb * x1
    End Sub

    Sub AreaCheck(ByRef kh As Integer, ByVal nmax As Integer, ByVal epsA As Single,
        ByRef xbuf() As Single, ByRef xbuf2() As Single, ByRef ybuf() As Single,
        ByRef ybuf2() As Single)
        '2面の重なりﾁｪｯｸ
        'kh･･･区分（0:関係なし、1:重なり有り）
        'nmax･･･面の節点数
        'epsA･･･面積誤差の基準
        'xbuf(),ybuf(),xbuf2(),ybuf2()･･･2面の節点座標
        Dim ic As Integer
        Dim area, x1, x2, xb, y1, y2, yb As Single
        Dim i, j As Integer

        kh = 0
        For i = 1 To nmax
            xb = xbuf(i)
            yb = ybuf(i)
            x1 = xbuf2(nmax)
            y1 = ybuf2(nmax)
            ic = 0
            For j = 1 To nmax - 1
                x2 = xbuf2(j)
                y2 = ybuf2(j)
                Call AreaCalc(area, x1, y1, x2, y2, xb, yb) '三角形の符号付き面積算出
                If area < epsA Then
                    ic = 1
                    Exit For
                Else
                    x1 = x2
                    y1 = y2
                End If
            Next j
            If ic = 0 Then
                kh = 1
                Exit For
            End If
        Next i

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single
        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = Xdata(k)
                        AxisMin(2) = Ydata(k)
                        AxisMin(3) = Zdata(k)
                        For ii = 1 To 3
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xdata(k)
                                Case 2
                                    b1 = Ydata(k)
                                Case 3
                                    b1 = Zdata(k)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = AxisMax(1) - AxisMin(1)
        For ii = 2 To 3
            b1 = AxisMax(ii) - AxisMin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisCheck2(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef xmin As Single, ByRef xmax As Single, _
    ByRef ymin As Single, ByRef ymax As Single)
        'ｽｸﾘｰﾝにおけるX、Y軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim ie, ik, k, j, nmax As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = Xscreen(k) : xmax = xmin
                        ymin = Yscreen(k) : ymax = ymin
                    Else
                        If Xscreen(k) < xmin Then
                            xmin = Xscreen(k)
                        ElseIf Xscreen(k) > xmax Then
                            xmax = Xscreen(k)
                        End If
                        If Yscreen(k) < ymin Then
                            ymin = Yscreen(k)
                        ElseIf Yscreen(k) > ymax Then
                            ymax = Yscreen(k)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisCheckM(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single, ByRef axmax() As Single, ByRef axmin() As Single)
        'ﾓﾃﾞﾙ座標X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        'axmax()･･･X,Y,Z座標の最大値
        'axmin()･･･X,Y,Z座標の最小値
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        axmin(1) = Xmodel(k)
                        axmin(2) = Ymodel(k)
                        axmin(3) = Zmodel(k)
                        For ii = 1 To 3
                            axmax(ii) = axmin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xmodel(k)
                                Case 2
                                    b1 = Ymodel(k)
                                Case 3
                                    b1 = Zmodel(k)
                            End Select
                            If b1 < axmin(ii) Then
                                axmin(ii) = b1
                            ElseIf b1 > axmax(ii) Then
                                axmax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = axmax(1) - axmin(1)
        For ii = 2 To 3
            b1 = axmax(ii) - axmin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisCheckSet(ByVal np As Integer, ByRef xrange() As Single, ByRef yrange() As Single,
        ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ（ｾｯﾄ）
        'np･･･節点数
        'xrange()･･･X座標最小値･最大値
        'yrange()･･･Y座標最小値･最大値
        'xdb(),ydb()･･･節点座標
        Dim i As Integer

        For i = 1 To np
            If i = 1 Then
                xrange(1) = xdb(1)
                xrange(2) = xdb(1)
                yrange(1) = ydb(1)
                yrange(2) = ydb(1)
            Else
                If xdb(i) < xrange(1) Then
                    xrange(1) = xdb(i)
                ElseIf xdb(i) > xrange(2) Then
                    xrange(2) = xdb(i)
                End If
                If ydb(i) < yrange(1) Then
                    yrange(1) = ydb(i)
                ElseIf ydb(i) > yrange(2) Then
                    yrange(2) = ydb(i)
                End If
            End If
        Next i

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single, ByRef zbuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ie, kind, nb As Integer
        Dim b2, rlenb As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        Call AxisCheck(1, Nelem, rlenb)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        b2 = rlenb * 0.00002
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i
        For i = 1 To Npoint
            If Xdata(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If Ydata(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
            If Zdata(i) <> 0 Then
                Call RoundTreat(kind, nb, zbuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        Dim i, ie, ii, j, jj, k, mx, nmax As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To nmax - 1
                i = NNelem(ie, ii)
                For jj = ii + 1 To nmax
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        mb = Ndim * (mx + 1)

    End Sub

    Sub BDatDupCheck(ByRef nd As Integer)
        '温度固定節点の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(NboundP) As Integer

        nd = 0
        For i = 1 To NboundP - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To NboundP
                    If NrestP(i) = NrestP(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            TempFixP(i, j) = TempFixP(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(NboundP) As Integer
            Dim buf2(NboundP, Nstep) As Single
            For i = 1 To NboundP
                nbuf1(i) = NrestP(i)
                For j = 1 To Nstep
                    buf2(i, j) = TempFixP(i, j)
                Next j
            Next i
            nb = NboundP - nd
            ReDim NrestP(nb), TempFixP(nb, Nstep)
            nb = 0
            For i = 1 To NboundP
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    NrestP(nb) = nbuf1(i)
                    For j = 1 To Nstep
                        TempFixP(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            NboundP = nb
            NboundPb = nb
        End If

    End Sub

    Sub BoundTempTreat()   '温度固定ﾃﾞｰﾀの処理
        Dim aa$
        Dim i, ic, ie, j, jj, jmax, k, kb, kmax, nb As Integer
        If KindElem = 0 Then
            jmax = 8
            kmax = 6
        End If
        Nbound = NboundP
        If NboundS > 0 Then
            Nbound = Nbound + jmax * NboundS
        End If
        If Nbound > 0 Then
            ReDim Nrest(Nbound), TempFix(Nbound, Nstep)
            k = 0
            If NboundP > 0 Then
                For i = 1 To NboundP
                    k = k + 1
                    Nrest(k) = NrestP(i)
                    For j = 1 To Nstep
                        TempFix(k, j) = TempFixP(i, j)
                    Next j
                Next i
            End If
            If NboundS > 0 Then
                Dim kbuf(8) As Integer
                For i = 1 To NboundS
                    ie = KsurfRest(i, 1)
                    kb = KsurfRest(i, 2)
                    If KindElem = 0 Then
                        Call SurfaceSet(ie, kb, kbuf)   '面の設定
                    End If
                    For j = 1 To jmax
                        k = k + 1
                        Nrest(k) = kbuf(j)
                        For jj = 1 To Nstep
                            TempFix(k, jj) = TempFixS(i, jj)
                        Next jj
                    Next j
                Next i
            End If
            '温度固定の節点重複の削除
            ic = 0
            For i = 1 To Nbound - 1
                For j = i + 1 To Nbound
                    If Nrest(i) = Nrest(j) Then
                        For jj = 1 To Nstep
                            If TempFix(i, jj) <> TempFix(j, jj) Then
                                aa$ = "節点No." + LTrim(Nrest(i)) + "で複数の固定温度が設定されています"
                                MsgBox(aa$, 0, "データエラー")
                                End
                            End If
                        Next jj
                        ic = 1
                        Nrest(j) = 0
                    End If
                Next j
            Next i
            If ic > 0 Then
                '温度固定節点の新規保存
                nb = 0
                For i = 1 To Nbound
                    If Nrest(i) > 0 Then
                        nb = nb + 1
                    End If
                Next i
                Dim nrb(nb) As Integer
                Dim tfb(nb, Nstep) As Single
                nb = 0
                For i = 1 To Nbound
                    If Nrest(i) > 0 Then
                        nb = nb + 1
                        nrb(nb) = Nrest(i)
                        For j = 1 To Nstep
                            tfb(nb, j) = TempFix(i, j)
                        Next j
                    End If
                Next i
                Nbound = nb
                ReDim Nrest(Nbound), TempFix(Nbound, Nstep)
                For i = 1 To Nbound
                    Nrest(i) = nrb(i)
                    For j = 1 To Nstep
                        TempFix(i, j) = tfb(i, j)
                    Next j
                Next i
            End If
        End If

    End Sub

    Sub BSDatDupCheck(ByRef nd As Integer)
        '温度固定境界面の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim ksbuf(NboundS) As Integer

        nd = 0
        For i = 1 To NboundS - 1
            If ksbuf(i) = 0 Then
                For ii = i + 1 To NboundS
                    If KsurfRest(i, 1) = KsurfRest(ii, 1) Then
                        If KsurfRest(i, 2) = KsurfRest(ii, 2) Then
                            nd = nd + 1
                            ksbuf(ii) = -1
                            For j = 1 To Nstep
                                TempFixS(i, j) = TempFixS(ii, j)
                            Next j
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(NboundS, 2) As Integer
            Dim buf2(NboundS, Nstep) As Single
            For i = 1 To NboundS
                For j = 1 To 2
                    nbuf1(i, j) = KsurfRest(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = TempFixS(i, j)
                Next j
            Next i
            nb = NboundS - nd
            ReDim KsurfRest(nb, 2), TempFixS(nb, Nstep)
            nb = 0
            For i = 1 To NboundS
                If ksbuf(i) = 0 Then
                    nb = nb + 1
                    KsurfRest(nb, 1) = nbuf1(i, 1)
                    KsurfRest(nb, 2) = nbuf1(i, 2)
                    For j = 1 To Nstep
                        TempFixS(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            NboundS = nb
            NboundSb = nb
        End If

    End Sub

    Sub CondTempCheck(ByRef ik As Integer, ByRef tempcmax As Single, _
        ByRef tempcmin As Single)
        '境界条件の温度ﾁｪｯｸ
        'ik･･･境界の有無（0:なし、1:有り）
        'tempcmax･･･最高温度
        'tempcmin･･･最低温度
        Dim i, ic, j As Integer
        Dim b1 As Single

        ik = 0
        ic = 0
        If NboundP > 0 Then
            ik = 1
            For i = 1 To NboundP
                For j = 1 To Nstep
                    b1 = TempFixP(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

        If NboundS > 0 Then
            ik = 1
            For i = 1 To NboundS
                For j = 1 To Nstep
                    b1 = TempFixS(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

        If Namb > 0 Then
            ik = 1
            For i = 1 To Namb
                For j = 1 To Nstep
                    b1 = TempAmb(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

        If Nrad > 0 Then
            ik = 1
            For i = 1 To Nrad
                For j = 1 To Nstep
                    b1 = TempRad(i, j)
                    Call TempComp(ic, b1, tempcmax, tempcmin) '温度比較
                Next j
            Next i
        End If

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Private Sub CrosPoint(ByRef alphab As Single, ByRef beta As Single, _
    ByVal eps As Single, ByVal xbuf() As Single, ByVal ybuf() As Single, _
    ByVal xbufc() As Single, ByVal ybufc() As Single)
        '2線分の交点計算
        'alphab,beta･･･各線分の交点位置比
        'xbuf(),ybuf()･･･基準側の2節点座標
        'xbufc(),ybufc()･･･比較側の2節点座標
        Dim a1, a2, b1, b2, b3, b4, bb, cc, dd, dist, distb As Single
        Dim eps2, f1, f2, xb, yb As Single

        alphab = -1
        eps2 = eps * eps
        a1 = ybuf(2) - ybuf(1)
        b1 = -(xbuf(2) - xbuf(1))
        dist = Math.Sqrt(a1 * a1 + b1 * b1)
        a2 = ybufc(2) - ybufc(1)
        b2 = -(xbufc(2) - xbufc(1))
        distb = Math.Sqrt(a2 * a2 + b2 * b2)
        If dist < eps Or distb < eps Then
            Exit Sub
        End If
        If Math.Abs(a1) < eps And Math.Abs(b2) < eps Then
            xb = xbufc(1)
            yb = ybuf(1)
            b1 = xb - xbuf(1)
            b2 = yb - ybufc(1)
            cc = b1 * (xb - xbuf(2))
            dd = b2 * (yb - ybufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        ElseIf Math.Abs(b1) < eps And Math.Abs(a2) < eps Then
            xb = xbuf(1)
            yb = ybufc(1)
            b1 = yb - ybuf(1)
            b2 = xb - xbufc(1)
            cc = b1 * (yb - ybuf(2))
            dd = b2 * (xb - xbufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        Else
            f1 = a1 * xbuf(1) + b1 * ybuf(1)
            f2 = a2 * xbufc(1) + b2 * ybufc(1)
            bb = a1 * b2 - b1 * a2
            If Math.Abs(bb) > eps2 Then
                xb = (f1 * b2 - f2 * b1) / bb
                yb = (f2 * a1 - f1 * a2) / bb
                b1 = xb - xbuf(1)
                b2 = yb - ybuf(1)
                cc = b1 * (xb - xbuf(2))
                dd = b2 * (yb - ybuf(2))
                If cc < eps2 And dd < eps2 Then
                    b3 = xb - xbufc(1)
                    b4 = yb - ybufc(1)
                    cc = b3 * (xb - xbufc(2))
                    dd = b4 * (yb - ybufc(2))
                    If cc < eps2 And dd < eps2 Then
                        alphab = Math.Sqrt(b1 * b1 + b2 * b2) / dist
                        beta = Math.Sqrt(b3 * b3 + b4 * b4) / distb
                    End If
                End If
            End If
        End If

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim a1$, aa$, aerr1$
        Dim ib, ie, im, ip, j, k, kb, kmax, n1, n2, nb, nmax, nx As Integer
        Dim b1, b2, b3 As Single
        Dim abuf(10) As String

        a1$ = ""
        aerr1$ = "になっています！"
        num = 0
        If KindElem = 0 Then
            kmax = 6
            nmax = 20
        End If

        If Npoint < 20 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Nmat < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf(num) = aa$ + "  材料データ数が" + Str$(Nmat) + aerr1$
        End If
        If Nstep < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf(num) = aa$ + "  時間ステップ数が" + Str$(Nstep) + aerr1$
        End If
        If NboundP < 1 And NboundS < 1 Then
            If Nflux < 1 And Namb < 1 And Nrad < 1 And Nqint < 1 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  境界条件が設定されていません！"
            End If
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Or nb > Nmat Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                abuf(num) = aa$ + " 材料番号が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = 1 To nmax
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "の"
                    abuf(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
                If j < nmax Then
                    For k = j + 1 To nmax
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                            abuf(num) = aa$ + " ２つの節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = Xdata(ip) - Xdata(j)
                b2 = Ydata(ip) - Ydata(j)
                b3 = Zdata(ip) - Zdata(j)
                If b1 = 0 And b2 = 0 And b3 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "が"
                    abuf(num) = aa$ + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '不要節点のﾁｪｯｸ
        Dim kbuf(Npoint) As Integer
        For ie = 1 To Nelem
            For j = 1 To 20
                kbuf(NNelem(ie, j)) = 1
            Next j
        Next ie
        For ip = 1 To Npoint
            If kbuf(ip) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が使われていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ip

        '要素共通辺のﾁｪｯｸ
        ReDim kbuf(Npoint)
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) = 0 Then
                    kbuf(ip) = 1    '頂点節点に指定
                End If
            Next j
            For j = 13 To 19 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) = 0 Then
                    kbuf(ip) = 1    '頂点節点に指定
                End If
            Next j
        Next ie
        For ie = 1 To Nelem
            For j = 2 To 8 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
            For j = 9 To 12
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
            For j = 14 To 20 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ie

        '材料ﾃﾞｰﾀのﾁｪｯｸ
        For im = 1 To Nmat
            nb = NMdiv(im)
            If nb < 1 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                abuf(num) = aa$ + " 物性区分数が" + Str$(nb) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            If Rou(im) <= 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  材料番号" + Str$(im) + "で"
                abuf(num) = aa$ + " 密度が" + Str$(Rou(im)) + aerr1$
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            For j = 1 To nb
                b1 = Cspec(im, j)
                If b1 <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                    abuf(num) = aa$ + " 比熱が" + Str$(b1) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                For k = 1 To 3
                    Select Case k
                        Case 1
                            a1$ = "λxが"
                            b1 = RlamdaX(im, j)
                        Case 2
                            a1$ = "λyが"
                            b1 = RlamdaY(im, j)
                        Case 3
                            a1$ = "λzが"
                            b1 = RlamdaZ(im, j)
                    End Select
                    If b1 <= 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                        abuf(num) = aa$ + a1$ + Str$(b1) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If

                Next
                If j > 1 Then
                    b1 = TempMat(im, j) - TempMat(im, j - 1)
                    If b1 <= 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  材料番号" + Str$(im) + "の区分" + Str$(j) + "で"
                        abuf(num) = aa$ + " 上限温度が上がっていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                End If
            Next j
        Next im

        If Nstep > 1 Then   'ｽﾃｯﾌﾟ時間ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nstep
                If TimeStep(ib) <= 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  ステップ番号" + Str$(ib) + "の"
                    abuf(num) = aa$ + " 時間が" + Str$(TimeStep(ib)) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If NboundP > 0 Then     '温度固定の節点ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To NboundP
                nb = NrestP(ib)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  温度固定節点の固定番号" + Str$(ib) + "の"
                    abuf(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If NboundS > 0 Then     '温度固定境界のﾁｪｯｸ
            For ib = 1 To NboundS
                kb = KsurfRest(ib, 1)
                If kb < 1 Or kb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  温度固定境界の面固定番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 要素番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                kb = KsurfRest(ib, 2)
                If kb < 1 Or kb > 6 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  温度固定境界の面固定番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 面番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        Application.DoEvents()
        Call InnerCheck2S()    '内面ﾁｪｯｸ
        Application.DoEvents()
        If Nflux > 0 Then   '熱流束境界のﾁｪｯｸ
            For ib = 1 To Nflux
                kb = KsurfFlux(ib, 1)
                If kb < 1 Or kb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 要素番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                j = KsurfFlux(ib, 2)
                If j < 1 Or j > kmax Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 面番号が" + Str$(j) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf KSelem(kb, j) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱流束境界の番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 内部面が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Namb > 0 Then    '熱伝達ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Namb
                kb = KsurfAmb(ib, 1)
                If kb < 1 Or kb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 要素番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                j = KsurfAmb(ib, 2)
                If j < 1 Or j > kmax Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 面番号が" + Str$(j) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf KSelem(kb, j) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱伝達データの番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 内部面が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nrad > 0 Then    '熱放射ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nrad
                kb = KsurfRad(ib, 1)
                If kb < 1 Or kb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 要素番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                j = KsurfRad(ib, 2)
                If j < 1 Or j > kmax Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 面番号が" + Str$(j) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                ElseIf KSelem(kb, j) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  熱放射データの番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 内部面が指定されています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If Nqint > 0 Then   '内部発熱ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nqint
                nb = Kelemq(ib)
                If nb < 1 Or nb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  内部発熱の番号" + Str$(ib) + "で"
                    abuf(num) = aa$ + " 要素番号が" + Str$(nb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If NradG > 0 Then    '熱放射ｸﾞﾙｰﾌﾟのﾁｪｯｸ
            For ib = 1 To NradG
                For k = 1 To NumRad(ib)
                    kb = MemRad(ib, k, 1)
                    If kb < 1 Or kb > Nelem Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱放射グループ" + Str$(ib) + "の面No." + Str(kb) + "で"
                        abuf(num) = aa$ + " 要素番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    j = MemRad(ib, k, 2)
                    If j < 1 Or j > 6 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱放射グループ" + Str$(ib) + "の面No." + Str(kb) + "で"
                        abuf(num) = aa$ + " 面番号が" + Str$(j) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    ElseIf KSelem(kb, j) > 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  熱放射グループ" + Str$(ib) + "の面No." + Str(kb) + "で"
                        abuf(num) = aa$ + " 内部面が指定されています！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next k
                For k = 1 To NumRad(ib) - 1
                    n1 = MemRad(ib, k, 1)
                    n2 = MemRad(ib, k, 2)
                    For kb = k + 1 To NumRad(ib)
                        If n1 = MemRad(ib, kb, 1) Then
                            If n2 = MemRad(ib, kb, 2) Then
                                num = num + 1
                                aa$ = "Error" + LTrim$(Str$(num))
                                aa$ = aa$ + "  熱放射グループ" + Str$(ib) + +"の面No." + Str(k) + "と"
                                abuf$(num) = aa$ + " 面No." + Str$(kb) + "が重複しています！"
                                If num = 10 Then
                                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                    Exit Sub
                                End If
                            End If
                        End If
                    Next kb
                Next k
            Next ib
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, j, jb, k, nmax As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "AnIsoTherm" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindElem = dat(1)
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nstep = dat(1)

        If KindElem = 0 Then
            nmax = 20
        Else
            aa$ = "六面体要素以外は使用できません！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NboundP = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NboundS = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nflux = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Namb = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nrad = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nqint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) = "熱放射グループの数" Then
            NradG = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
        Else
            NradG = 0
        End If
        Ngaus = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        InitData = dat(1)

        ReDim NNelem(Nelem, nmax), NumMat(Nelem)
        ReDim Xdata(Npoint), Ydata(Npoint), Zdata(Npoint)
        ReDim NMdiv(Nmat), Rou(Nmat), TimeStep(Nstep)
        ReDim NrestP(NboundP), TempFixP(NboundP, Nstep)
        ReDim KsurfRest(NboundS, 2), TempFixS(NboundS, Nstep)
        ReDim KsurfFlux(Nflux, 2), Pnorm(Nflux, Nstep)
        ReDim KsurfAmb(Namb, 2), Alpha(Namb, Nstep), TempAmb(Namb, Nstep)
        ReDim KsurfRad(Nrad, 2), RadRate(Nrad, Nstep), TempRad(Nrad, Nstep)
        ReDim Kelemq(Nqint), Qint(Nqint, Nstep)
        ReDim NumRad(NradG)
        ReDim TempInit(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 10
                NNelem(i, j) = dat(j + 1)
            Next j
            If nmax > 10 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 11 To nmax
                    NNelem(i, j) = dat(j - 11)
                Next j
            End If
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(i) = dat(1)
            Ydata(i) = dat(2)
            Zdata(i) = dat(3)
        Next i

        aa$ = sr.ReadLine
        NMdivMax = 0
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NMdiv(i) = dat(1)
            If NMdiv(i) > NMdivMax Then
                NMdivMax = NMdiv(i)
            End If
        Next i
        ReDim Cspec(Nmat, NMdivMax), TempMat(Nmat, NMdivMax)
        ReDim RlamdaX(Nmat, NMdivMax)
        ReDim RlamdaY(Nmat, NMdivMax)
        ReDim RlamdaZ(Nmat, NMdivMax)
        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Rou(i) = dat(1)
            aa$ = sr.ReadLine
            For j = 1 To NMdiv(i)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Cspec(i, j) = dat(1)
                RlamdaX(i, j) = dat(2)
                RlamdaY(i, j) = dat(3)
                RlamdaZ(i, j) = dat(4)
                TempMat(i, j) = dat(5)
            Next j
        Next i

        If Nstep > 1 Then
            aa$ = sr.ReadLine
            For i = 1 To Nstep
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TimeStep(i) = dat(1)
            Next i
        End If

        If NboundP > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To NboundP
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NrestP(i) = dat(1)
                For j = 1 To Nstep
                    TempFixP(i, j) = dat(j + 1)
                Next j
            Next i
        End If

        If NboundS > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To NboundS
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KsurfRest(i, 1) = dat(1)
                KsurfRest(i, 2) = dat(2)
                For j = 1 To Nstep
                    TempFixS(i, j) = dat(j + 2)
                Next j
            Next i
        End If

        If Nflux > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nflux
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KsurfFlux(i, 1) = dat(1)
                KsurfFlux(i, 2) = dat(2)
                For j = 1 To Nstep
                    Pnorm(i, j) = dat(j + 2)
                Next j
            Next i
        End If

        If Namb > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Namb
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KsurfAmb(i, 1) = dat(1)
                KsurfAmb(i, 2) = dat(2)
                For j = 1 To Nstep
                    jb = 2 * j
                    Alpha(i, j) = dat(jb + 1)
                    TempAmb(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If Nrad > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nrad
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KsurfRad(i, 1) = dat(1)
                KsurfRad(i, 2) = dat(2)
                For j = 1 To Nstep
                    jb = 2 * j
                    RadRate(i, j) = dat(jb + 1)
                    TempRad(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If Nqint > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nqint
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Kelemq(i) = dat(1)
                For j = 1 To Nstep
                    Qint(i, j) = dat(j + 1)
                Next j
            Next i
        End If

        If NradG > 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            SigmaG = dat(1)
            aa$ = sr.ReadLine
            For i = 1 To NradG
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NumRad(i) = dat(1)
            Next i
            NumRmax = NumRad(1)
            If NradG > 1 Then
                For i = 2 To NradG
                    If NumRad(i) > NumRmax Then
                        NumRmax = NumRad(i)
                    End If
                Next
            End If
            ReDim MemRad(NradG, NumRmax, 2), EpsRad(NradG, NumRmax)
            For i = 1 To NradG
                aa$ = sr.ReadLine
                aa$ = sr.ReadLine
                For k = 1 To NumRad(i)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To 2
                        MemRad(i, k, j) = dat(j)
                    Next j
                    EpsRad(i, k) = dat(3)
                Next k
            Next i
        End If

        If InitData > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Npoint
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TempInit(i) = dat(1)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j, k As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "AnIsoTherm"
        sw.WriteLine(aa$)
        aa$ = "要素種類（0:六面体、1:四面体）" + "," + Str(KindElem)
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)
        aa$ = "境界条件のステップ数" + "," + Str(Nstep)
        sw.WriteLine(aa$)
        aa$ = "温度固定節点数" + "," + Str(NboundP)
        sw.WriteLine(aa$)
        aa$ = "温度固定境界の面の数" + "," + Str(NboundS)
        sw.WriteLine(aa$)
        aa$ = "熱流束境界の面の数" + "," + Str(Nflux)
        sw.WriteLine(aa$)
        aa$ = "熱伝達境界の面の数" + "," + Str(Namb)
        sw.WriteLine(aa$)
        aa$ = "熱放射境界の面の数" + "," + Str(Nrad)
        sw.WriteLine(aa$)
        aa$ = "内部発熱の要素の数" + "," + Str(Nqint)
        sw.WriteLine(aa$)

        aa$ = "熱放射グループの数" + "," + Str(NradG)
        sw.WriteLine(aa$)

        Ngaus = 3 : Ndim = 1 : Mprop = 5
        aa$ = "ガウス積分点数" + "," + Str(Ngaus)
        sw.WriteLine(aa$)
        aa$ = "１節点当りの変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)
        aa$ = "材料データの成分数" + "," + Str(Mprop)
        sw.WriteLine(aa$)
        aa$ = "初期値データの有無" + "," + Str(InitData)
        sw.WriteLine(aa$)

        If KindElem = 0 Then
            sw.WriteLine("要素データ (要素番号、材料番号、20節点番号)")
        End If
        For i = 1 To Nelem
            aa$ = Str$(i) + "," + Str$(NumMat(i))
            For j = 1 To 10
                aa$ = aa$ + "," + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
            If KindElem = 0 Then
                aa$ = "    " + Str$(NNelem(i, 11))
                For j = 12 To 20
                    aa$ = aa$ + "," + Str$(NNelem(i, j))
                Next j
                sw.WriteLine(aa$)
            End If
        Next i

        aa$ = "節点データ（節点番号、Ｘ座標、Ｙ座標、Ｚ座標）"
        sw.WriteLine(aa$)
        For i = 1 To Npoint
            aa$ = Str$(i)
            aa$ = aa$ + ",  " + Str$(Xdata(i))
            aa$ = aa$ + ",  " + Str$(Ydata(i))
            aa$ = aa$ + ",  " + Str$(Zdata(i))
            sw.WriteLine(aa$)
        Next i

        sw.WriteLine("各材料の物性区分数（材料番号、区分数)")
        For i = 1 To Nmat
            aa$ = "  " + Str$(i) + ",  " + Str$(NMdiv(i))
            sw.WriteLine(aa$)
        Next i
        sw.WriteLine("各材料の物性データ")
        For i = 1 To Nmat
            aa$ = "材料番号" + "," + Str(i)
            sw.WriteLine(aa$)
            aa$ = "  密度" + "," + Str(Rou(i))
            sw.WriteLine(aa$)
            sw.WriteLine("  区分番号、比熱、λx、λy、λz、上限温度")
            For j = 1 To NMdiv(i)
                aa$ = "    " + Str$(j) + ",  " + Str$(Cspec(i, j))
                aa$ = aa$ + ",  " + Str$(RlamdaX(i, j))
                aa$ = aa$ + ",  " + Str$(RlamdaY(i, j))
                aa$ = aa$ + ",  " + Str$(RlamdaZ(i, j))
                aa$ = aa$ + ",  " + Str$(TempMat(i, j))
                sw.WriteLine(aa$)
            Next j
        Next i

        If Nstep > 1 Then
            sw.WriteLine("境界条件のステップ時間（ステップ番号、時間）")
            For i = 1 To Nstep
                aa$ = Str$(i) + ",  " + Str$(TimeStep(i))
                sw.WriteLine(aa$)
            Next i
        End If

        If NboundP > 0 Then
            sw.WriteLine("温度固定節点データ (節点固定番号、節点番号、各ｽﾃｯﾌﾟの温度)")
            For i = 1 To NboundP
                aa$ = Str$(i) + ",  " + Str$(NrestP(i))
                For j = 1 To Nstep
                    aa$ = aa$ + ",  " + Str$(TempFixP(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If NboundS > 0 Then
            aa$ = "温度固定境界データ（面固定番号、要素番号、面番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの温度）"
            sw.WriteLine(aa$)
            For i = 1 To NboundS
                aa$ = Str$(i)
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(KsurfRest(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",  " + Str$(TempFixS(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nflux > 0 Then
            aa$ = "熱流束データ（熱流束番号、要素番号、面番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの熱流束（単位面積当り））"
            sw.WriteLine(aa$)
            For i = 1 To Nflux
                aa$ = Str$(i)
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(KsurfFlux(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",  " + Str$(Pnorm(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Namb > 0 Then
            aa$ = "熱伝達データ（熱伝達番号、要素番号、面番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの熱伝達係数・外部温度）"
            sw.WriteLine(aa$)
            For i = 1 To Namb
                aa$ = Str$(i)
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(KsurfAmb(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",  " + Str$(Alpha(i, j))
                    aa$ = aa$ + ",  " + Str$(TempAmb(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nrad > 0 Then
            aa$ = "熱放射データ（熱放射番号、要素番号、面番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの放射係数・外部温度）"
            sw.WriteLine(aa$)
            For i = 1 To Nrad
                aa$ = Str$(i)
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(KsurfRad(i, j))
                Next j
                For j = 1 To Nstep
                    aa$ = aa$ + ",  " + Str$(RadRate(i, j))
                    aa$ = aa$ + ",  " + Str$(TempRad(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If Nqint > 0 Then
            aa$ = "内部発熱データ（内部発熱番号、要素番号、"
            aa$ = aa$ + "各ｽﾃｯﾌﾟの内部発熱率）"
            sw.WriteLine(aa$)
            For i = 1 To Nqint
                aa$ = Str$(i) + ",  " + Str$(Kelemq(i))
                For j = 1 To Nstep
                    aa$ = aa$ + ",  " + Str$(Qint(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        If NradG > 0 Then
            aa$ = "熱放射グループ共通のステファン・ボルツマン定数" + ",  " + Str(SigmaG)
            sw.WriteLine(aa$)
            aa$ = "熱放射グループ（グループ番号、面の個数）"
            sw.WriteLine(aa$)
            For i = 1 To NradG
                aa$ = Str(i) + ",  " + Str(NumRad(i))
                sw.WriteLine(aa$)
            Next i
            For i = 1 To NradG
                aa = "＜グループ＞" + Str(i)
                sw.WriteLine(aa$)
                aa$ = "  面No.、要素番号、要素内面番号、放射率"
                sw.WriteLine(aa$)
                For k = 1 To NumRad(i)
                    aa$ = "  " + Str$(k)
                    For j = 1 To 2
                        aa$ = aa$ + ",  " + Str$(MemRad(i, k, j))
                    Next j
                    aa$ = aa$ + ",  " + Str(EpsRad(i, k))
                    sw.WriteLine(aa$)
                Next k
            Next i
        End If


        If InitData > 0 Then
            aa$ = "初期値データ（節点番号、初期温度）"
            sw.WriteLine(aa$)
            For i = 1 To Npoint
                aa$ = Str$(i) + ",  " + Str$(TempInit(i))
                sw.WriteLine(aa$)
            Next i
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Private Sub DistComp(ByRef ic As Integer, ByVal kbuf() As Integer)
        '距離比較
        'ic･･･基準面の距離区分（0:近い、1:遠い）
        'kbuf()･･･比較面の8節点番号
        Dim i, i1, i2, ib, icc, j, j1, j2, jb, nmax As Integer
        Dim alphab, b1, b2, b3, beta, c1, dist, distc, eps, xb, yb, zb As Single
        Dim xbuf(2), xbufc(2), ybuf(2), ybufc(2) As Single

        If KindElem = 0 Then
            nmax = 8
            c1 = 0.25
        End If
        ic = -1  '仮設定
        For i = 1 To nmax - 1 Step 2
            i1 = KsurfPoint(i)
            ib = i + 2
            If ib > nmax Then
                ib = 1
            End If
            i2 = KsurfPoint(ib)
            xbuf(1) = Xscreen(i1)
            ybuf(1) = Yscreen(i1)
            xbuf(2) = Xscreen(i2)
            ybuf(2) = Yscreen(i2)
            b1 = Xmodel(i1) - Xmodel(i2)
            b2 = Ymodel(i1) - Ymodel(i2)
            b3 = Zmodel(i1) - Zmodel(i2)
            eps = 0.0001 * Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
            For j = 1 To nmax - 1 Step 2
                j1 = kbuf(j)
                jb = j + 2
                If jb > nmax Then
                    jb = 1
                End If
                j2 = kbuf(jb)
                icc = 0
                If i1 = j1 Or i1 = j2 Then
                    icc = 1
                ElseIf i2 = j1 Or i2 = j2 Then
                    icc = 1
                End If
                If icc = 0 Then
                    xbufc(1) = Xscreen(j1)
                    ybufc(1) = Yscreen(j1)
                    xbufc(2) = Xscreen(j2)
                    ybufc(2) = Yscreen(j2)
                    Call CrosPoint(alphab, beta, eps, xbuf, ybuf, xbufc, ybufc)
                    '2線分の交点計算

                    If alphab >= 0 Then
                        b1 = 1 - alphab
                        xb = b1 * Xmodel(i1) + alphab * Xmodel(i2)
                        yb = b1 * Ymodel(i1) + alphab * Ymodel(i2) + Rlenmodel
                        zb = b1 * Zmodel(i1) + alphab * Zmodel(i2) - Eyeh
                        dist = xb * xb + yb * yb + zb * zb
                        b1 = 1 - beta
                        xb = b1 * Xmodel(j1) + beta * Xmodel(j2)
                        yb = b1 * Ymodel(j1) + beta * Ymodel(j2) + Rlenmodel
                        zb = b1 * Zmodel(j1) + beta * Zmodel(j2) - Eyeh
                        distc = xb * xb + yb * yb + zb * zb
                        If dist < distc Then
                            ic = 0
                        Else
                            ic = 1
                        End If
                        icc = -1
                        Exit For
                    End If
                End If
            Next j
            If icc < 0 Then
                Exit For
            End If
        Next i

        If ic < 0 Then
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = KsurfPoint(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            dist = xb * xb + yb * yb + zb * zb
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = kbuf(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            distc = xb * xb + yb * yb + zb * zb

            If dist < distc Then
                ic = 0
            Else
                ic = 1
            End If
        End If

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal kind As Integer, _
    ByVal aa As String, ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'kind･･･種別（0:基本、1:灰色）
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        If kind = 0 Then
            gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)
        Else
            gr.DrawString(aa$, fnt2, Brushes.Gray, xb, yb)
        End If

    End Sub

    Sub DrawScale(ByVal dpixb As Single, ByVal dpiyb As Single, _
    ByVal xlen As Single, ByVal ylen As Single, ByVal xmin As Single, _
    ByVal xmax As Single, ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        If Cfix < 1 Then
            If b1 <> 0 Then
                cx = xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2 / Cfix
            End If
        Else
            If b1 <> 0 Then
                cx = Cfix * xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2
            End If
        End If
        If b1 = 0 Then
            XYscale = cy
        ElseIf b2 = 0 Then
            XYscale = cx
        ElseIf cy < cx Then
            XYscale = cy
        Else
            XYscale = cx
        End If

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        KnumDisp = 0
        KdispENum = 0
        KdispMat = 0
        KdispPNum = 0
        KdispSNum = 0

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemRelation(ByRef icc As Integer, ByVal ie As Integer, ByVal ib As Integer)
        '要素関係の判別
        'icc･･･関係の有無（0:なし、1:有り）
        'ie･･･元の要素番号
        'ib･･･対象の要素番号
        Dim i, j, k As Integer
        If KindElem = 0 Then
            For i = 1 To 7 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
            For i = 13 To 19 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
        End If

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ic, ie, j, k, kmin, kmax, nmax As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    b1 = Xscreen(k) - Xmouse(1)
                    b2 = Xscreen(k) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(k) - Ymouse(2)
                        b2 = Yscreen(k) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = nmax Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub ElemSearch3(ByRef nd As Integer)
        '矩形領域の外周面の探索
        'k1,k2･･･要素の開始、終了番号
        'nb･･･適合面の数
        Dim ie, j, jmax, k, kb, kk, kmax, nb As Integer
        Dim b1, b2 As Single
        If KindElem = 0 Then
            jmax = 6
            kmax = 8
        End If
        For ie = 1 To Nelem
            For j = 1 To jmax
                KAppSide(ie, j) = 0
            Next j
        Next ie
        nd = 0
        For nb = 1 To NSdraw
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KEInside(ie, kb) = 0 Then
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)     '面の設定
                End If
                For k = 1 To kmax
                    kk = KsurfPoint(k)
                    b1 = Xscreen(kk) - Xmouse(1)
                    b2 = Xscreen(kk) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(kk) - Ymouse(2)
                        b2 = Yscreen(kk) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If k = kmax Then
                                KAppSide(ie, kb) = 1
                                nd = nd + 1
                            End If
                        Else
                            Exit For
                        End If
                    Else
                        Exit For
                    End If
                Next k
            End If
        Next nb

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub EyeCheck(ByVal k1 As Integer, ByVal k2 As Integer)
        '視線ﾁｪｯｸ（面の中心位置で判定）
        'k1,k2･･･開始・終了要素番号
        Dim ie, j, k, kb, kmax, nb, nmax As Integer
        Dim b1, b2, b3, b4, xb, yb, zb As Single
        Dim eyevec(3), planevec(3) As Single
        Dim xbufb(3), ybufb(3), zbuf(3) As Single

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
            nb = 4
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If

                        xb = 0 : yb = 0 : zb = 0
                        For j = 2 To nmax Step 2
                            kb = KsurfPoint(j)
                            xb = xb + Xmodel(kb)
                            yb = yb + Ymodel(kb)
                            zb = zb + Zmodel(kb) - Eyeh
                            Select Case j
                                Case 2
                                    xbufb(2) = Xmodel(kb)
                                    ybufb(2) = Ymodel(kb)
                                    zbuf(2) = Zmodel(kb) - Eyeh
                                Case 4
                                    xbufb(3) = Xmodel(kb)
                                    ybufb(3) = Ymodel(kb)
                                    zbuf(3) = Zmodel(kb) - Eyeh
                            End Select
                        Next j
                        xbufb(1) = xb / nb
                        ybufb(1) = yb / nb
                        zbuf(1) = zb / nb

                        '逆視線ﾍﾞｸﾄﾙの計算
                        b1 = -xbufb(1)
                        b2 = -(Rlenmodel + ybufb(1))
                        b3 = -zbuf(1)
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        eyevec(1) = b1 / b4
                        eyevec(2) = b2 / b4
                        eyevec(3) = b3 / b4

                        '面の法線ﾍﾞｸﾄﾙの計算
                        b1 = (ybufb(2) - ybufb(1)) * (zbuf(3) - zbuf(1))
                        b1 = b1 - (zbuf(2) - zbuf(1)) * (ybufb(3) - ybufb(1))
                        b2 = (zbuf(2) - zbuf(1)) * (xbufb(3) - xbufb(1))
                        b2 = b2 - (xbufb(2) - xbufb(1)) * (zbuf(3) - zbuf(1))
                        b3 = (xbufb(2) - xbufb(1)) * (ybufb(3) - ybufb(1))
                        b3 = b3 - (ybufb(2) - ybufb(1)) * (xbufb(3) - xbufb(1))
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        planevec(1) = b1 / b4
                        planevec(2) = b2 / b4
                        planevec(3) = b3 / b4

                        '逆視線ﾍﾞｸﾄﾙと面法線ﾍﾞｸﾄﾙのｽｶﾗｰ積計算
                        b1 = 0
                        For j = 1 To 3
                            b1 = b1 + eyevec(j) * planevec(j)
                        Next j
                        If b1 <= 0 Then
                            KSelem(ie, k) = -1   '視線外
                        End If
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Function FreeMemoryGet() As Single
        '空きﾒﾓﾘｰ容量の取得
        'freemem･･･ﾒﾓﾘｰ空き容量（KB）
        Dim memfree As New System.Diagnostics.PerformanceCounter

        memfree.CategoryName = "Memory"
        memfree.CounterName = "Available KBytes"
        Return memfree.NextValue()

    End Function
    Sub HDatDupCheck(ByRef nd As Integer)
        '熱流束の重複ﾁｪｯｸ･修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nflux) As Integer

        nd = 0
        For i = 1 To Nflux - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nflux
                    If KsurfFlux(i, 1) = KsurfFlux(ii, 1) Then
                        If KsurfFlux(i, 2) = KsurfFlux(ii, 2) Then
                            nd = nd + 1
                            kbuf(ii) = -1
                            For j = 1 To Nstep
                                Pnorm(i, j) = Pnorm(ii, j)
                            Next j
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nflux, 2) As Integer
            Dim buf2(Nflux, Nstep) As Single
            For i = 1 To Nflux
                For j = 1 To 2
                    nbuf1(i, j) = KsurfFlux(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = Pnorm(i, j)
                Next j
            Next i
            nb = Nflux - nd
            ReDim KsurfFlux(nb, 2), Pnorm(nb, Nstep)
            nb = 0
            For i = 1 To Nflux
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 2
                        KsurfFlux(nb, j) = nbuf1(i, j)
                    Next j
                    For j = 1 To Nstep
                        Pnorm(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            Nflux = nb
            Nfluxb = nb
        End If

    End Sub

    Sub InnerCheck2(ByVal k1 As Integer, ByVal k2 As Integer)
        '内面ﾁｪｯｸ
        'k1,k2･･･開始・終了要素番号
        Dim ib, ic, icc, ie, ik, ik1, j, jj, jx, k, kk, kmax, ksb, ksmax, m, nmax As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            ksmax = 4
            nmax = 7
            ReDim KinnerLine(12 * Nelem) '内部辺のﾌﾗｯｸﾞ
        End If
        Dim ksbuf(Nelem, kmax) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        For ie = k1 To k2 - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If k = 1 Then
                        icc = 0
                    End If
                    If ksbuf(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If
                        ik = 0
                        For ib = ie + 1 To k2
                            If KdrawElem(ib) > 0 Then
                                If icc = 0 Then
                                    Call ElemRelation(icc, ie, ib)    '要素関係の判別
                                End If
                                If icc > 0 Then
                                    For kk = 1 To kmax
                                        If ksbuf(ib, kk) = 0 Then
                                            If KindElem = 0 Then
                                                Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                            End If

                                            ik1 = KsurfPoint(1)
                                            ic = 0
                                            For j = 1 To nmax Step 2
                                                If ik1 = kbuf(j) Then
                                                    jx = j
                                                    For jj = 3 To nmax Step 2
                                                        jx = jx - 2
                                                        If jx < 1 Then
                                                            jx = nmax
                                                        End If
                                                        If KsurfPoint(jj) <> kbuf(jx) Then
                                                            ic = 1
                                                            Exit For
                                                        End If
                                                    Next jj
                                                    If ic = 0 Then
                                                        ik = 1
                                                        KSelem(ie, k) = 1
                                                        KSelem(ib, kk) = 1
                                                        ksbuf(ie, k) = 1
                                                        ksbuf(ib, kk) = 1
                                                        For m = 1 To ksmax
                                                            ksb = KSideLine(ie, k, m)
                                                            KinnerLine(ksb) = 1
                                                            ksb = KSideLine(ib, kk, m)
                                                            KinnerLine(ksb) = 1
                                                        Next m
                                                    End If
                                                    Exit For
                                                End If
                                            Next j
                                            If ik = 1 Then
                                                Exit For
                                            End If
                                        End If
                                    Next kk
                                    If ik = 1 Then
                                        Exit For
                                    End If
                                End If
                            Else
                                icc = 0
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Private Sub InnerCheck2S()
        '内面ﾁｪｯｸ
        Dim ib, ic, ie, ik, ik1, j, jj, jx, k, kk, kmax, nmax As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
        End If
        Dim ksbuf(Nelem, kmax) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        ReDim KSelem(Nelem, kmax)
        For ie = 1 To Nelem
            For k = 1 To kmax
                If ksbuf(ie, k) = 0 Then
                    If KindElem = 0 Then
                        Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                    End If
                    ik = 0
                    For ib = ie + 1 To Nelem
                        For kk = 1 To kmax
                            If ksbuf(ib, kk) = 0 Then
                                If KindElem = 0 Then
                                    Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                End If
                                ik1 = KsurfPoint(1)
                                ic = 0
                                For j = 1 To nmax
                                    If ik1 = kbuf(j) Then
                                        jx = j
                                        For jj = 2 To nmax
                                            jx = jx - 1
                                            If jx < 1 Then
                                                jx = nmax
                                            End If
                                            If KsurfPoint(jj) <> kbuf(jx) Then
                                                ic = 1
                                                Exit For
                                            End If
                                        Next jj
                                        If ic = 0 Then
                                            ik = 1
                                            KSelem(ie, k) = 1
                                            KSelem(ib, kk) = 1
                                            ksbuf(ie, k) = 1
                                            ksbuf(ib, kk) = 1
                                            Exit For
                                        End If
                                    End If
                                Next j
                                If ik = 1 Then
                                    Exit For
                                End If
                            End If
                        Next kk
                        If ik = 1 Then
                            Exit For
                        End If
                    Next ib
                End If
            Next k
        Next ie

    End Sub

    Sub InsideBorderCorrect(ByRef ic As Integer)
        '内部面境界の設定削除
        'ic･･･設定削除の有無（0:なし、1:有り）
        Dim aa$
        Dim ib, j, jb, k, kb, nb As Integer

        ic = 0
        Call InnerCheck2S()    '内面ﾁｪｯｸ2S
        Application.DoEvents()

        If NboundS > 0 Then     '温度固定境界のﾁｪｯｸ
            nb = 0
            For ib = 1 To NboundS
                kb = KsurfRest(ib, 1)
                jb = KsurfRest(ib, 2)
                If KSelem(kb, jb) > 0 Then
                    nb = nb + 1
                End If
            Next ib
            If nb > 0 Then
                ic = 1
                nb = 0
                For ib = 1 To NboundS
                    kb = KsurfRest(ib, 1)
                    jb = KsurfRest(ib, 2)
                    If KSelem(kb, jb) = 0 Then
                        nb = nb + 1
                        For j = 1 To 2
                            KsurfRest(nb, j) = KsurfRest(ib, j)
                        Next j
                        For j = 1 To Nstep
                            TempFixS(nb, j) = TempFixS(ib, j)
                        Next j
                    End If
                Next ib
                NboundS = nb
            End If
        End If

        If Nflux > 0 Then   '熱流束境界のﾁｪｯｸ
            nb = 0
            For ib = 1 To Nflux
                kb = KsurfFlux(ib, 1)
                jb = KsurfFlux(ib, 2)
                If KSelem(kb, jb) > 0 Then
                    nb = nb + 1
                End If
            Next ib
            If nb > 0 Then
                ic = 1
                nb = 0
                For ib = 1 To Nflux
                    kb = KsurfFlux(ib, 1)
                    jb = KsurfFlux(ib, 2)
                    If KSelem(kb, jb) = 0 Then
                        nb = nb + 1
                        For j = 1 To 2
                            KsurfFlux(nb, j) = KsurfFlux(ib, j)
                        Next j
                        For j = 1 To Nstep
                            Pnorm(nb, j) = Pnorm(ib, j)
                        Next j
                    End If
                Next ib
                Nflux = nb
            End If
        End If

        If Namb > 0 Then    '熱伝達ﾃﾞｰﾀのﾁｪｯｸ
            nb = 0
            For ib = 1 To Namb
                kb = KsurfAmb(ib, 1)
                jb = KsurfAmb(ib, 2)
                If KSelem(kb, jb) > 0 Then
                    nb = nb + 1
                End If
            Next ib
            If nb > 0 Then
                ic = 1
                nb = 0
                For ib = 1 To Namb
                    kb = KsurfAmb(ib, 1)
                    jb = KsurfAmb(ib, 2)
                    If KSelem(kb, jb) = 0 Then
                        nb = nb + 1
                        For j = 1 To 2
                            KsurfAmb(nb, j) = KsurfAmb(ib, j)
                        Next j
                        For j = 1 To Nstep
                            Alpha(nb, j) = Alpha(ib, j)
                            TempAmb(nb, j) = TempAmb(ib, j)
                        Next j
                    End If
                Next ib
                Namb = nb
            End If
        End If

        If Nrad > 0 Then    '熱放射ﾃﾞｰﾀのﾁｪｯｸ
            nb = 0
            For ib = 1 To Nrad
                kb = KsurfRad(ib, 1)
                jb = KsurfRad(ib, 2)
                If KSelem(kb, jb) > 0 Then
                    nb = nb + 1
                End If
            Next ib
            If nb > 0 Then
                ic = 1
                nb = 0
                For ib = 1 To Nrad
                    kb = KsurfRad(ib, 1)
                    jb = KsurfRad(ib, 2)
                    If KSelem(kb, jb) = 0 Then
                        nb = nb + 1
                        For j = 1 To 2
                            KsurfRad(nb, j) = KsurfRad(ib, j)
                        Next j
                        For j = 1 To Nstep
                            RadRate(nb, j) = RadRate(ib, j)
                            TempRad(nb, j) = TempRad(ib, j)
                        Next j
                    End If
                Next ib
                Nrad = nb
            End If
        End If

        If NradG > 0 Then    '熱放射ｸﾞﾙｰﾌﾟのﾁｪｯｸ
            For ib = 1 To NradG
                nb = 0
                For k = 1 To NumRad(ib)
                    kb = MemRad(ib, k, 1)
                    jb = MemRad(ib, k, 2)
                    If KSelem(kb, jb) > 0 Then
                        nb = nb + 1
                    End If
                Next k
                If nb > 0 Then
                    ic = 1
                    nb = 0
                    For k = 1 To NumRad(ib)
                        kb = MemRad(ib, k, 1)
                        jb = MemRad(ib, k, 2)
                        If KSelem(kb, jb) = 0 Then
                            nb = nb + 1
                            For j = 1 To 2
                                MemRad(ib, nb, j) = MemRad(ib, k, j)
                            Next j
                            EpsRad(ib, nb) = EpsRad(ib, k)
                        End If
                    Next k

                    NumRad(ib) = nb
                End If
            Next ib
        End If

        If ic = 0 Then
            aa$ = "内部面境界はありませんでした！"
            MsgBox(aa$, 0, "チェック結果")
        Else
            aa$ = "内部面境界を設定削除しました！"
            MsgBox(aa$, 0, "チェック結果")
        End If

    End Sub

    Sub JacobElem(ByVal ie As Integer, ByVal kgaus As Integer, _
    ByVal numg As Integer, ByRef dj As Double)
        '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ie･･･要素番号
        'kgaus･･･ｶﾞｳｽ点座標計算の有無（0:なし、1:有り）
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        Dim c1$, c2$
        Dim i, ii, j, nmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            nmax = 20
        End If

        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        If kgaus > 0 Then
            XG(numg) = 0
            YG(numg) = 0
            ZG(numg) = 0
            For ii = 1 To nmax
                XG(numg) = XG(numg) + XE(ii) * Shape(ii)
                YG(numg) = YG(numg) + YE(ii) * Shape(ii)
                ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
            Next ii
        End If
        TG(numg) = 0
        For ii = 1 To nmax
            TG(numg) = TG(numg) + TE(ii) * Shape(ii)
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ii
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)
        b1 = RJmat(1, 1) * RinvJ(1, 1) + RJmat(1, 2) * RinvJ(2, 1)
        dj = b1 + RJmat(1, 3) * RinvJ(3, 1)
        If dj <= 0 Then
            c1$ = "計算実行エラー"
            c2$ = "要素" + LTrim$(Str$(ie)) + "（ガウス点" + LTrim$(Str$(numg))
            c2$ = c2$ + "）でヤコビアン体積が負か０になりました。"
            MsgBox(c2$, 0, c1$)
            End
        Else
            '    本来の逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽは下記のようにdjで割った値である。
            '        For i = 1 To 3
            '            For j = 1 To 3
            '                RinvJ(i, j) = RinvJ(i, j) / dj
            '            Next j
            '        Next i
            For ii = 1 To nmax
                For i = 1 To 3
                    b1 = RinvJ(i, 1) * Ds(ii) + RinvJ(i, 2) * Dt(ii)
                    b1 = (b1 + RinvJ(i, 3) * Du(ii)) / dj
                    Select Case i
                        Case 1
                            Dx(ii) = b1
                        Case 2
                            Dy(ii) = b1
                        Case 3
                            Dz(ii) = b1
                    End Select
                Next i
            Next ii
        End If
    End Sub

    Sub JacobElem2(ByVal ie As Integer, ByVal numg As Integer, _
    ByRef dj As Double, ByVal knode() As Integer)
        '要素表面のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ie･･･要素番号
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        'knode()･･･節点の順位
        Dim i, ii, ip, j, kmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            kmax = 8
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        '    XG(numg) = 0
        '    YG(numg) = 0
        '    ZG(numg) = 0
        '    For ip = 1 To 8
        '        ii = knode(ip)
        '        XG(numg) = XG(numg) + XE(ii) * Shape(ii)
        '        YG(numg) = YG(numg) + YE(ii) * Shape(ii)
        '        ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
        '    Next ip
        For ip = 1 To kmax
            ii = knode(ip)
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ip

        '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの配列に、面積分で使用するための値を保存
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)

    End Sub

    Sub JacobElem2M(ByVal numg As Integer, ByVal knode() As Integer, ByRef shapeb() As Double,
        ByRef xeb() As Single, ByRef yeb() As Single, ByRef zeb() As Single,
        ByRef xgb() As Double, ByRef ygb() As Double, ByRef zgb() As Double,
        ByRef dsb() As Double, ByRef dtb() As Double, ByRef dub() As Double)
        '要素表面のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        'knode()･･･節点の順位
        'shapeb()･･･形状関数
        'xeb(),yeb(),zeb()･･･節点のX,Y,Z座標
        'xgb(),ygb(),zgb()･･･ｶﾞｳｽ点のX,Y,Z座標
        'dsb(),dtb(),dub()･･･s,t,uの偏微分
        Dim i, ii, ip, j, kmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            kmax = 8
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        xgb(numg) = 0
        ygb(numg) = 0
        zgb(numg) = 0
        For ip = 1 To kmax
            ii = knode(ip)
            xgb(numg) = xgb(numg) + xeb(ii) * shapeb(ii)
            ygb(numg) = ygb(numg) + yeb(ii) * shapeb(ii)
            zgb(numg) = zgb(numg) + zeb(ii) * shapeb(ii)
        Next ip
        For ip = 1 To kmax
            ii = knode(ip)
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = xeb(ii)
                    Case 2
                        b1 = yeb(ii)
                    Case 3
                        b1 = zeb(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + dsb(ii) * b1
                RJmat(2, j) = RJmat(2, j) + dtb(ii) * b1
                RJmat(3, j) = RJmat(3, j) + dub(ii) * b1
            Next j
        Next ip

        '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの配列に、面積分で使用するための値を保存
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)

    End Sub

    Function KSideLine(ByVal ib As Integer, ByVal kb As Integer,
    ByVal kk As Integer) As Integer
        '辺番号の照合
        'ib･･･要素番号
        'kb･･･面番号
        'kk･･･面の辺番号
        Dim ksb As Integer

        If KindElem = 0 Then    '六面体
            Select Case kb
                Case 1
                    ksb = kk
                Case 2
                    ksb = 13 - kk
                Case 3
                    Select Case kk
                        Case 1
                            ksb = 5
                        Case 2
                            ksb = 9
                        Case 3
                            ksb = 6
                        Case 4
                            ksb = 1
                    End Select
                Case 4
                    Select Case kk
                        Case 1
                            ksb = 7
                        Case 2
                            ksb = 11
                        Case 3
                            ksb = 8
                        Case 4
                            ksb = 3
                    End Select
                Case 5
                    Select Case kk
                        Case 1
                            ksb = 6
                        Case 2
                            ksb = 10
                        Case 3
                            ksb = 7
                        Case 4
                            ksb = 2
                    End Select
                Case 6
                    Select Case kk
                        Case 1
                            ksb = 8
                        Case 2
                            ksb = 12
                        Case 3
                            ksb = 5
                        Case 4
                            ksb = 4
                    End Select
            End Select
            KSideLine = 12 * (ib - 1) + ksb
        End If

    End Function

    Sub MDatSet2(ByVal nm1 As Integer, ByVal nm2 As Integer)
        '材料ﾃﾞｰﾀの保存変更2
        'nm1･･･初期の材料数
        'nm2･･･変更後の材料数（同じか増加）
        Dim i, j, nmmaxb As Integer
        Dim nmbuf(nm1) As Integer
        Dim roubuf(nm1) As Single
        nmmaxb = 1
        For i = 1 To nm1
            nmbuf(i) = NMdiv(i)
            If NMdiv(i) > nmmaxb Then
                nmmaxb = NMdiv(i)
            End If
        Next i
        Dim csbuf(nm1, nmmaxb) As Single
        Dim rlxbuf(nm1, nmmaxb) As Single
        Dim rlybuf(nm1, nmmaxb) As Single
        Dim rlzbuf(nm1, nmmaxb) As Single
        Dim tmbuf(nm1, nmmaxb) As Single
        For i = 1 To nm1
            roubuf(i) = Rou(i)
            For j = 1 To NMdiv(i)
                csbuf(i, j) = Cspec(i, j)
                rlxbuf(i, j) = RlamdaX(i, j)
                rlybuf(i, j) = RlamdaY(i, j)
                rlzbuf(i, j) = RlamdaZ(i, j)
                tmbuf(i, j) = TempMat(i, j)
            Next j
        Next i

        NMdivMax = nmmaxb
        ReDim NMdiv(nm2), Rou(nm2)
        ReDim Cspec(nm2, NMdivMax), TempMat(nm2, NMdivMax)
        ReDim RlamdaX(nm2, NMdivMax)
        ReDim RlamdaY(nm2, NMdivMax)
        ReDim RlamdaZ(nm2, NMdivMax)
        For i = 1 To nm1
            NMdiv(i) = nmbuf(i)
            Rou(i) = roubuf(i)
            For j = 1 To NMdiv(i)
                Cspec(i, j) = csbuf(i, j)
                RlamdaX(i, j) = rlxbuf(i, j)
                RlamdaY(i, j) = rlybuf(i, j)
                RlamdaZ(i, j) = rlzbuf(i, j)
                TempMat(i, j) = tmbuf(i, j)
            Next j
        Next i
        If nm2 > nm1 Then
            For i = nm1 + 1 To nm2
                NMdiv(i) = 1
            Next i
        End If

    End Sub

    Sub MiniSort2(ByVal kbuf() As Integer, ByRef nsort() As Integer, ByVal work() As Single, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ﾐﾆｿｰﾄ2
        '同じ変数値の場合に、第2座標値でソート
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'nsort()･･･元の変数番号のｿｰﾄ順番号
        'work()･･･ｿｰﾄ変数値
        'xbuf(),ybuf()･･･節点の座標
        Dim i, j, jb, jx, k, k1, k2, kx, nb, n1, n2, nx As Integer
        Dim b1 As Single
        Dim nsbuf(10), nsort2(10), nsort3(10) As Integer
        Dim work2(10), work3(10) As Single

        i = 1
        Do
            If work(i) = work(i + 1) Then
                b1 = work(i)
                n1 = i
                For j = i + 1 To Npoint
                    If b1 = work(j) Then
                        n2 = j
                    Else
                        Exit For
                    End If
                Next j
                nx = n2 - n1 + 1
                ReDim nsort2(nx), work2(nx)
                jb = 0
                For j = n1 To n2
                    jb = jb + 1
                    nsort2(jb) = jb
                    nb = nsort(j)
                    Select Case kbuf(2)
                        Case 1
                            work2(jb) = xbuf(nb)
                        Case 2
                            work2(jb) = ybuf(nb)
                        Case 3
                            work2(jb) = zbuf(nb)
                    End Select
                Next j
                Call ShellSortStd(nx, nsort2, work2)    'ｼｪｯﾙｿｰﾄ基本

                k = 1
                Do
                    If work2(k) = work2(k + 1) Then
                        b1 = work2(k)
                        k1 = k
                        For j = k + 1 To nx
                            If b1 = work2(j) Then
                                k2 = j
                            Else
                                Exit For
                            End If
                        Next j
                        kx = k2 - k1 + 1
                        ReDim nsort3(kx), work3(kx), nsbuf(kx)
                        jb = 0
                        For j = k1 To k2
                            jb = jb + 1
                            nsbuf(jb) = nsort2(j)
                            nsort3(jb) = jb
                            nb = nsort(n1 + nsort2(j) - 1)
                            Select Case kbuf(3)
                                Case 1
                                    work3(jb) = xbuf(nb)
                                Case 2
                                    work3(jb) = ybuf(nb)
                                Case 3
                                    work3(jb) = zbuf(nb)
                            End Select
                        Next j
                        Call ShellSortStd(kx, nsort3, work3)    'ｼｪｯﾙｿｰﾄ基本
                        For j = 1 To kx
                            jb = k1 + j - 1
                            nsort2(jb) = nsbuf(nsort3(j))
                        Next j
                        k = k2 + 1
                    Else
                        k = k + 1
                    End If
                Loop While k < nx

                For j = 1 To nx
                    jb = n1 + nsort2(j) - 1
                    nsort2(j) = nsort(jb)
                Next j
                For j = 1 To nx
                    jx = n1 + j - 1
                    nsort(jx) = nsort2(j)
                Next j
                i = n2 + 1
            Else
                i = i + 1
            End If
        Loop While i < Npoint

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MousePointSearch(ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の節点の探索
        'kp･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        Dim ie, k, kb, kk, kmax, nb As Integer
        Dim b1, b2, b3, rlen2, x2, y2 As Single
        Dim kbuf(Npoint) As Integer

        If KindElem = 0 Then
            kmax = 8
        End If
        kp = 0
        For nb = NSdraw To 1 Step -1
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
            End If
            For k = 1 To kmax
                kk = KsurfPoint(k)
                If kbuf(kk) = 0 Then
                    kbuf(kk) = 1
                    x2 = Xscreen(kk)
                    y2 = Yscreen(kk)
                    Call XYCalc(x2, y2)     'X,Yの作画座標計算
                    b1 = x2 - xb
                    b2 = y2 - yb
                    b3 = b1 * b1 + b2 * b2
                    If kp = 0 Or b3 < rlen2 Then
                        kp = kk
                        rlen2 = b3
                    End If
                End If
            Next k
        Next nb

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\AnIsoTMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            For i = 1 To 3
                Koption(i) = Val(sr.ReadLine)
            Next i
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\AnIsoTMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 3
            sw.WriteLine(Koption(i))
        Next i
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub OuterReset(ByVal k1 As Integer, ByVal k2 As Integer)
        '外郭面の再設定
        'k1,k2･･･開始・終了要素番号
        Dim ie, k, kmax As Integer

        If KindElem = 0 Then
            kmax = 6
        End If
        For ie = k1 To k2
            For k = 1 To kmax
                If KSelem(ie, k) < 0 Then
                    KSelem(ie, k) = 0
                End If
            Next k
        Next ie
    End Sub

    Sub OutSideCheck()
        '外周面のﾁｪｯｸ
        Dim ib, ic, ie, ik, ik1, j, jj, jmax, jx, k, kk, kmax As Integer
        Dim kbuf(8) As Integer
        ReDim KEInside(Nelem, 6)

        If KindElem = 0 Then
            jmax = 8
            kmax = 6
        End If
        For ie = 1 To Nelem - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KEInside(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If
                        ik = 0
                        For ib = ie + 1 To Nelem
                            If KdrawElem(ib) > 0 Then
                                For kk = 1 To kmax
                                    If KEInside(ib, kk) = 0 Then
                                        If KindElem = 0 Then
                                            Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                        End If
                                        ik1 = KsurfPoint(1)
                                        ic = 0
                                        For j = 1 To jmax
                                            If ik1 = kbuf(j) Then
                                                jx = j
                                                For jj = 2 To jmax
                                                    jx = jx - 1
                                                    If jx < 1 Then
                                                        jx = jmax
                                                    End If
                                                    If KsurfPoint(jj) <> kbuf(jx) Then
                                                        ic = 1
                                                        Exit For
                                                    End If
                                                Next jj
                                                If ic = 0 Then
                                                    ik = 1
                                                    KEInside(ie, k) = 1
                                                    KEInside(ib, kk) = 1
                                                End If
                                                Exit For
                                            End If
                                        Next j
                                        If ik = 1 Then
                                            Exit For
                                        End If
                                    End If
                                Next kk
                                If ik = 1 Then
                                    Exit For
                                End If
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByVal rlenscreen As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        Dim ib, ie, j, nmax As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / Zview(ib)
                        Xscreen(ib) = b1 * Xview(ib)
                        Yscreen(ib) = b1 * (Yview(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie
    End Sub

    Sub PlaneCutElem()
        '面切断の要素選択
        Dim ic, ie, j, k, nmax As Integer
        Dim b1, std As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        If KE3DCut(2) > 0 Then
            std = VE3DCut(1) - VE3DCut(2)
        Else
            std = VE3DCut(1) + VE3DCut(2)
        End If

        For ie = 1 To Nelem
            If KdrawElem(ie) > 0 Then
                ic = 0
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    Select Case KE3DCut(1)
                        Case 1
                            b1 = Xdata(k)
                        Case 2
                            b1 = Ydata(k)
                        Case 3
                            b1 = Zdata(k)
                    End Select
                    If KE3DCut(2) > 0 Then
                        If b1 < std Then
                            ic = 1
                        End If
                    Else
                        If b1 > std Then
                            ic = 1
                        End If
                    End If
                    If ic > 0 Then
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PnumImproveNew(ByRef mb As Integer, ByVal mbx As Integer,
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        '節点順序の改良
        'mb･･･改良後のﾊﾞﾝﾄﾞ幅
        'mbx･･･改良前のﾊﾞﾝﾄﾞ幅
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ic, ie, k, k1, k2, mbb As Integer
        Dim rlenb As Single
        Dim kbuf(3), kbufb(3), kcbuf(Npoint) As Integer
        Dim rlbuf(3) As Single

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie
        Call AxisCheck(1, Nelem, rlenb)     'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        mbb = 0
        For k = 1 To 3
            kbuf(1) = k
            Select Case k
                Case 1
                    k1 = 2
                    k2 = 3
                Case 2
                    k1 = 3
                    k2 = 1
                Case 3
                    k1 = 1
                    k2 = 2
            End Select
            rlbuf(k1) = AxisMax(k1) - AxisMin(k1)
            rlbuf(k2) = AxisMax(k2) - AxisMin(k2)
            If rlbuf(k1) > rlbuf(k2) Then
                kbuf(2) = k1
                kbuf(3) = k2
            Else
                kbuf(2) = k2
                kbuf(3) = k1
            End If

            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            ic = 0
            If k = 1 Or mb < mbb Then
                ic = 1
            End If
            If ic > 0 Then
                For i = 1 To 3
                    kbufb(i) = kbuf(i)
                Next i
                mbb = mb
            End If
        Next k

        If mbb < mbx Then
            For i = 1 To 3
                kbuf(i) = kbufb(i)
            Next i
            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            For i = 1 To Npoint
                Kcomp(i) = kcbuf(i)
            Next i
        Else
            mb = mbx
        End If

        For i = 1 To Npoint
            ib = Kcomp(i)
            KRcomp(ib) = i
        Next i

    End Sub

    Sub PointSearch(ByRef nd As Integer)
        '矩形領域の節点の探索
        'nb･･･適合節点の数
        Dim i, ie, k, kb, kk, nb, nmax As Integer
        Dim b1, b2 As Single

        If KindElem = 0 Then
            nmax = 8
            ReDim KinnerLine(12 * Nelem) '内部辺のﾌﾗｯｸﾞ
        End If
        For i = 1 To Npoint
            KAppPoint(i) = 0
        Next i
        nd = 0
        For nb = 1 To NSdraw
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
            End If
            For k = 1 To nmax
                kk = KsurfPoint(k)
                If KAppPoint(kk) = 0 Then
                    b1 = Xscreen(kk) - Xmouse(1)
                    b2 = Xscreen(kk) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(kk) - Ymouse(2)
                        b2 = Yscreen(kk) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            KAppPoint(kk) = 1
                            nd = nd + 1
                        End If
                    End If
                End If
            Next k
        Next nb

    End Sub

    Sub QDatDupCheck(ByRef nd As Integer)
        '内部発熱の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nqint) As Integer

        nd = 0
        For i = 1 To Nqint - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nqint
                    If Kelemq(i) = Kelemq(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Nstep
                            Qint(i, j) = Qint(ii, j)
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nqint) As Integer
            Dim buf2(Nqint, Nstep) As Single
            For i = 1 To Nqint
                nbuf1(i) = Kelemq(i)
                For j = 1 To Nstep
                    buf2(i, j) = Qint(i, j)
                Next j
            Next i
            nb = Nqint - nd
            ReDim Kelemq(nb), Qint(nb, Nstep)
            nb = 0
            For i = 1 To Nqint
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    Kelemq(nb) = nbuf1(i)
                    For j = 1 To Nstep
                        Qint(nb, j) = buf2(i, j)
                    Next j
                End If
            Next i
            Nqintb = nb
            Nqint = nb
        End If

    End Sub

    Sub RangeCheck(ByRef kh As Integer, ByVal epsL As Single, ByRef xrange() As Single,
        ByRef xrange2() As Single, ByRef yrange() As Single, ByRef yrange2() As Single)
        '範囲ﾁｪｯｸ
        'kh･･･判定区分（0:判定なし、1:別範囲）
        'epsL･･･微小誤差長さ
        kh = 0
        If xrange(2) < xrange2(1) + epsL Or xrange2(2) < xrange(1) + epsL Then
            kh = 1
        ElseIf yrange(2) < yrange2(1) + epsL Or yrange2(2) < yrange(1) + epsL Then
            kh = 1
        End If
    End Sub

    Sub RDatDupCheck(ByRef nd As Integer)
        '熱放射の重複ﾁｪｯｸ･修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(Nrad) As Integer

        nd = 0
        For i = 1 To Nrad - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To Nrad
                    If KsurfRad(i, 1) = KsurfRad(ii, 1) Then
                        If KsurfRad(i, 2) = KsurfRad(ii, 2) Then
                            nd = nd + 1
                            kbuf(ii) = -1
                            For j = 1 To Nstep
                                RadRate(i, j) = RadRate(ii, j)
                                TempRad(i, j) = TempRad(ii, j)
                            Next j
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nrad, 2) As Integer
            Dim buf2(Nrad, Nstep), buf3(Nrad, Nstep) As Single
            For i = 1 To Nrad
                For j = 1 To 2
                    nbuf1(i, j) = KsurfRad(i, j)
                Next j
                For j = 1 To Nstep
                    buf2(i, j) = RadRate(i, j)
                    buf3(i, j) = TempRad(i, j)
                Next j
            Next i
            nb = Nrad - nd
            ReDim KsurfRad(nb, 2), RadRate(nb, Nstep), TempRad(nb, Nstep)
            nb = 0
            For i = 1 To Nrad
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    For j = 1 To 2
                        KsurfRad(nb, j) = nbuf1(i, j)
                    Next j
                    For j = 1 To Nstep
                        RadRate(nb, j) = buf2(i, j)
                        TempRad(nb, j) = buf3(i, j)
                    Next j
                End If
            Next i
            Nrad = nb
            Nradb = nb
        End If

    End Sub

    Sub RGdatAddG(ByVal ngb As Integer)
        '熱放射ｸﾞﾙｰﾌﾟのｸﾞﾙｰﾌﾟ追加
        'ngb･･･元のｸﾞﾙｰﾌﾟ数
        Dim i, j, k, num0, numb As Integer
        Dim nbuf1(ngb), nbuf2(ngb, NumRmax, 2) As Integer
        Dim buf1(ngb), buf3(ngb, NumRmax) As Single

        If ngb > 0 Then
            num0 = NumRmax
            For i = 1 To ngb
                nbuf1(i) = NumRad(i)
                For k = 1 To NumRad(i)
                    For j = 1 To 2
                        nbuf2(i, k, j) = MemRad(i, k, j)
                    Next j
                    buf3(i, k) = EpsRad(i, k)
                Next k
            Next i
            numb = NumRad(1)
            For i = 2 To NradG
                If NumRad(i) > numb Then
                    numb = NumRad(i)
                End If
            Next i
            NumRmax = numb
            If numb < num0 Then
                num0 = numb
            End If
            ReDim NumRad(NradG), MemRad(NradG, NumRmax, 2), EpsRad(NradG, NumRmax)
            For i = 1 To ngb
                NumRad(i) = nbuf1(i)
                For k = 1 To NumRad(i)
                    For j = 1 To 2
                        MemRad(i, k, j) = nbuf2(i, k, j)
                    Next j
                    EpsRad(i, k) = buf3(i, k)
                Next k
            Next i
        Else
            NumRmax = 2
            ReDim NumRad(NradG), MemRad(NradG, NumRmax, 2), EpsRad(NradG, NumRmax)
            For i = 1 To NradG
                NumRad(i) = 2
            Next i
        End If

    End Sub

    Sub RGdatAddG2(ByVal k0 As Integer, ByRef mbuf() As Integer, ByVal epsb As Single)
        '熱放射ｸﾞﾙｰﾌﾟの面の追加
        'k0･･･ｸﾞﾙｰﾌﾟNo.
        'mbuf()･･･面の要素番号・面番号
        'epsb･･･放射率
        Dim i, j, k, num0, numb As Integer
        Dim nbuf1(NradG), nbuf2(NradG, NumRmax, 2) As Integer
        Dim buf1(NradG), buf3(NradG, NumRmax) As Single

        num0 = NumRmax
        For i = 1 To NradG
            nbuf1(i) = NumRad(i)
            For k = 1 To NumRad(i)
                For j = 1 To 2
                    nbuf2(i, k, j) = MemRad(i, k, j)
                Next j
                buf3(i, k) = EpsRad(i, k)
            Next k
        Next i
        numb = NumRad(k0) + 1
        For i = 1 To NradG
            If NumRad(i) > numb Then
                numb = NumRad(i)
            End If
        Next i
        NumRmax = numb
        If numb < num0 Then
            num0 = numb
        End If
        ReDim NumRad(NradG), MemRad(NradG, NumRmax, 2), EpsRad(NradG, NumRmax)
        For i = 1 To NradG
            NumRad(i) = nbuf1(i)
            For k = 1 To NumRad(i)
                For j = 1 To 2
                    MemRad(i, k, j) = nbuf2(i, k, j)
                Next j
                EpsRad(i, k) = buf3(i, k)
            Next k
            If i = k0 Then
                k = NumRad(i) + 1
                NumRad(i) = k
                For j = 1 To 2
                    MemRad(i, k, j) = mbuf(j)
                Next j
                EpsRad(i, k) = epsb
            End If
        Next i

    End Sub

    Sub RadGDupCheck(ByRef nd As Integer)
        '熱放射ｸﾞﾙｰﾌﾟの重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ic, j, k, kk, mx, nb As Integer
        Dim kbuf(NradG, NumRmax), mbuf(2) As Integer

        nd = 0
        For i = 1 To NradG
            For k = 1 To NumRad(i) - 1
                If kbuf(i, k) = 0 Then
                    mbuf(1) = MemRad(i, k, 1)
                    mbuf(2) = MemRad(i, k, 2)
                    For kk = k + 1 To NumRad(i)
                        If kbuf(i, kk) = 0 Then
                            If mbuf(1) = MemRad(i, kk, 1) And mbuf(2) = MemRad(i, kk, 2) Then
                                kbuf(i, kk) = -1
                                nd = nd + 1
                            End If
                        End If
                    Next kk
                End If
            Next k
        Next i

        If nd > 0 Then
            For i = 1 To NradG
                mx = 0
                For k = 1 To NumRad(i)
                    If kbuf(i, k) = 0 Then
                        mx = mx + 1
                        For j = 1 To 2
                            MemRad(i, mx, j) = MemRad(i, k, j)
                        Next j
                        EpsRad(i, mx) = EpsRad(i, k)
                    End If
                Next k
                NumRad(i) = mx
            Next i
            ic = 0
            For i = 1 To NradG
                If NumRad(i) < 1 Then
                    ic = 1
                End If
            Next i
            If ic > 0 Then
                nb = 0
                For i = 1 To NradG
                    If NumRad(i) > 0 Then
                        nb = nb + 1
                        For k = 1 To NumRad(i)
                            For j = 1 To 2
                                MemRad(nb, k, j) = MemRad(i, k, j)
                            Next j
                            EpsRad(nb, k) = EpsRad(i, k)
                        Next k
                        NumRad(nb) = NumRad(i)
                    End If
                Next i
                NradG = nb
            End If
        End If

        NradGb = NradG

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub Rounding(ByVal kind As Integer, ByVal kfnum As Integer, _
        ByRef temp As Single)
        '丸め処理
        'kind･･･丸め種類（0:下丸め、1:上丸め）
        'kfnum･･･小数点以下の桁数
        'temp･･･温度
        Dim b1, b2, b3, b4 As Single
        b1 = 10 ^ kfnum
        b3 = temp * b1
        b4 = Int(b3 / b1)
        b2 = b3 - b1 * b4
        If b2 = 0 Then
            temp = CInt(b3) / b1
        ElseIf kind > 0 Then
            temp = Int(b3 + 1) / b1
        Else
            temp = Int(b3) / b1
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub ShapeFunc(ByVal kind As Integer, ByVal s As Double, ByVal t As Double, _
    ByVal u As Double)
        '形状関数の計算
        'kind･･･計算区分（0:基本、1:偏微分込み）
        's･･･要素のξ座標
        't･･･要素のη座標
        'u･･･要素のζ座標
        Dim s1, s1m, s2m, t1, t1m, t2m, u1, u1m, u2m As Double
        s1 = 1 + s
        s1m = 1 - s
        s2m = 1 - s * s
        t1 = 1 + t
        t1m = 1 - t
        t2m = 1 - t * t
        u1 = 1 + u
        u1m = 1 - u
        u2m = 1 - u * u
        Shape(1) = s1m * t1m * u1 * (-s - t + u - 2) / 8
        Shape(2) = s2m * t1m * u1 / 4
        Shape(3) = s1 * t1m * u1 * (s - t + u - 2) / 8
        Shape(4) = s1 * t2m * u1 / 4
        Shape(5) = s1 * t1 * u1 * (s + t + u - 2) / 8
        Shape(6) = s2m * t1 * u1 / 4
        Shape(7) = s1m * t1 * u1 * (-s + t + u - 2) / 8
        Shape(8) = s1m * t2m * u1 / 4
        Shape(9) = s1m * t1m * u2m / 4
        Shape(10) = s1 * t1m * u2m / 4
        Shape(11) = s1 * t1 * u2m / 4
        Shape(12) = s1m * t1 * u2m / 4
        Shape(13) = s1m * t1m * u1m * (-s - t - u - 2) / 8
        Shape(14) = s2m * t1m * u1m / 4
        Shape(15) = s1 * t1m * u1m * (s - t - u - 2) / 8
        Shape(16) = s1 * t2m * u1m / 4
        Shape(17) = s1 * t1 * u1m * (s + t - u - 2) / 8
        Shape(18) = s2m * t1 * u1m / 4
        Shape(19) = s1m * t1 * u1m * (-s + t - u - 2) / 8
        Shape(20) = s1m * t2m * u1m / 4

        If kind > 0 Then
            Ds(1) = t1m * u1 * (s + t - u + 1 + s) / 8
            Ds(2) = -t1m * u1 * s / 2
            Ds(3) = t1m * u1 * (s - t + u - 1 + s) / 8
            Ds(4) = t2m * u1 / 4
            Ds(5) = t1 * u1 * (s + t + u - 1 + s) / 8
            Ds(6) = -t1 * u1 * s / 2
            Ds(7) = t1 * u1 * (s - t - u + 1 + s) / 8
            Ds(8) = -t2m * u1 / 4
            Ds(9) = -t1m * u2m / 4
            Ds(10) = t1m * u2m / 4
            Ds(11) = t1 * u2m / 4
            Ds(12) = -t1 * u2m / 4
            Ds(13) = t1m * u1m * (s + t + u + 1 + s) / 8
            Ds(14) = -t1m * u1m * s / 2
            Ds(15) = t1m * u1m * (s - t - u - 1 + s) / 8
            Ds(16) = t2m * u1m / 4
            Ds(17) = t1 * u1m * (s + t - u - 1 + s) / 8
            Ds(18) = -t1 * u1m * s / 2
            Ds(19) = t1 * u1m * (s - t + u + 1 + s) / 8
            Ds(20) = -t2m * u1m / 4
            Dt(1) = s1m * u1 * (s + t - u + 1 + t) / 8
            Dt(2) = -s2m * u1 / 4
            Dt(3) = s1 * u1 * (-s + t - u + 1 + t) / 8
            Dt(4) = -s1 * u1 * t / 2
            Dt(5) = s1 * u1 * (s + t + u - 1 + t) / 8
            Dt(6) = s2m * u1 / 4
            Dt(7) = s1m * u1 * (-s + t + u - 1 + t) / 8
            Dt(8) = -s1m * u1 * t / 2
            Dt(9) = -s1m * u2m / 4
            Dt(10) = -s1 * u2m / 4
            Dt(11) = s1 * u2m / 4
            Dt(12) = s1m * u2m / 4
            Dt(13) = s1m * u1m * (s + t + u + 1 + t) / 8
            Dt(14) = -s2m * u1m / 4
            Dt(15) = s1 * u1m * (-s + t + u + 1 + t) / 8
            Dt(16) = -s1 * u1m * t / 2
            Dt(17) = s1 * u1m * (s + t - u - 1 + t) / 8
            Dt(18) = s2m * u1m / 4
            Dt(19) = s1m * u1m * (-s + t - u - 1 + t) / 8
            Dt(20) = -s1m * u1m * t / 2
            Du(1) = s1m * t1m * (-s - t + u - 1 + u) / 8
            Du(2) = s2m * t1m / 4
            Du(3) = s1 * t1m * (s - t + u - 1 + u) / 8
            Du(4) = s1 * t2m / 4
            Du(5) = s1 * t1 * (s + t + u - 1 + u) / 8
            Du(6) = s2m * t1 / 4
            Du(7) = s1m * t1 * (-s + t + u - 1 + u) / 8
            Du(8) = s1m * t2m / 4
            Du(9) = -s1m * t1m * u / 2
            Du(10) = -s1 * t1m * u / 2
            Du(11) = -s1 * t1 * u / 2
            Du(12) = -s1m * t1 * u / 2
            Du(13) = s1m * t1m * (s + t + u + 1 + u) / 8
            Du(14) = -s2m * t1m / 4
            Du(15) = s1 * t1m * (-s + t + u + 1 + u) / 8
            Du(16) = -s1 * t2m / 4
            Du(17) = s1 * t1 * (-s - t + u + 1 + u) / 8
            Du(18) = -s2m * t1 / 4
            Du(19) = s1m * t1 * (s - t + u + 1 + u) / 8
            Du(20) = -s1m * t2m / 4
        End If
    End Sub

    Sub ShapeFunc2(ByVal kind As Integer, ByVal s As Double, ByVal t As Double,
    ByVal u As Double, ByRef shapeb() As Double, ByRef dsb() As Double,
    ByRef dtb() As Double, ByRef dub() As Double)
        '形状関数の計算
        'kind･･･計算区分（0:基本、1:偏微分込み）
        's･･･要素のξ座標
        't･･･要素のη座標
        'u･･･要素のζ座標
        'shapeb()･･･形状関数
        'dsb(),dtb(),dub()･･･s,t,uの偏微分
        Dim s1, s1m, s2m, t1, t1m, t2m, u1, u1m, u2m As Double
        s1 = 1 + s
        s1m = 1 - s
        s2m = 1 - s * s
        t1 = 1 + t
        t1m = 1 - t
        t2m = 1 - t * t
        u1 = 1 + u
        u1m = 1 - u
        u2m = 1 - u * u
        shapeb(1) = s1m * t1m * u1 * (-s - t + u - 2) / 8
        shapeb(2) = s2m * t1m * u1 / 4
        shapeb(3) = s1 * t1m * u1 * (s - t + u - 2) / 8
        shapeb(4) = s1 * t2m * u1 / 4
        shapeb(5) = s1 * t1 * u1 * (s + t + u - 2) / 8
        shapeb(6) = s2m * t1 * u1 / 4
        shapeb(7) = s1m * t1 * u1 * (-s + t + u - 2) / 8
        shapeb(8) = s1m * t2m * u1 / 4
        shapeb(9) = s1m * t1m * u2m / 4
        shapeb(10) = s1 * t1m * u2m / 4
        shapeb(11) = s1 * t1 * u2m / 4
        shapeb(12) = s1m * t1 * u2m / 4
        shapeb(13) = s1m * t1m * u1m * (-s - t - u - 2) / 8
        shapeb(14) = s2m * t1m * u1m / 4
        shapeb(15) = s1 * t1m * u1m * (s - t - u - 2) / 8
        shapeb(16) = s1 * t2m * u1m / 4
        shapeb(17) = s1 * t1 * u1m * (s + t - u - 2) / 8
        shapeb(18) = s2m * t1 * u1m / 4
        shapeb(19) = s1m * t1 * u1m * (-s + t - u - 2) / 8
        shapeb(20) = s1m * t2m * u1m / 4

        If kind > 0 Then
            dsb(1) = t1m * u1 * (s + t - u + 1 + s) / 8
            dsb(2) = -t1m * u1 * s / 2
            dsb(3) = t1m * u1 * (s - t + u - 1 + s) / 8
            dsb(4) = t2m * u1 / 4
            dsb(5) = t1 * u1 * (s + t + u - 1 + s) / 8
            dsb(6) = -t1 * u1 * s / 2
            dsb(7) = t1 * u1 * (s - t - u + 1 + s) / 8
            dsb(8) = -t2m * u1 / 4
            dsb(9) = -t1m * u2m / 4
            dsb(10) = t1m * u2m / 4
            dsb(11) = t1 * u2m / 4
            dsb(12) = -t1 * u2m / 4
            dsb(13) = t1m * u1m * (s + t + u + 1 + s) / 8
            dsb(14) = -t1m * u1m * s / 2
            dsb(15) = t1m * u1m * (s - t - u - 1 + s) / 8
            dsb(16) = t2m * u1m / 4
            dsb(17) = t1 * u1m * (s + t - u - 1 + s) / 8
            dsb(18) = -t1 * u1m * s / 2
            dsb(19) = t1 * u1m * (s - t + u + 1 + s) / 8
            dsb(20) = -t2m * u1m / 4
            dtb(1) = s1m * u1 * (s + t - u + 1 + t) / 8
            dtb(2) = -s2m * u1 / 4
            dtb(3) = s1 * u1 * (-s + t - u + 1 + t) / 8
            dtb(4) = -s1 * u1 * t / 2
            dtb(5) = s1 * u1 * (s + t + u - 1 + t) / 8
            dtb(6) = s2m * u1 / 4
            dtb(7) = s1m * u1 * (-s + t + u - 1 + t) / 8
            dtb(8) = -s1m * u1 * t / 2
            dtb(9) = -s1m * u2m / 4
            dtb(10) = -s1 * u2m / 4
            dtb(11) = s1 * u2m / 4
            dtb(12) = s1m * u2m / 4
            dtb(13) = s1m * u1m * (s + t + u + 1 + t) / 8
            dtb(14) = -s2m * u1m / 4
            dtb(15) = s1 * u1m * (-s + t + u + 1 + t) / 8
            dtb(16) = -s1 * u1m * t / 2
            dtb(17) = s1 * u1m * (s + t - u - 1 + t) / 8
            dtb(18) = s2m * u1m / 4
            dtb(19) = s1m * u1m * (-s + t - u - 1 + t) / 8
            dtb(20) = -s1m * u1m * t / 2
            dub(1) = s1m * t1m * (-s - t + u - 1 + u) / 8
            dub(2) = s2m * t1m / 4
            dub(3) = s1 * t1m * (s - t + u - 1 + u) / 8
            dub(4) = s1 * t2m / 4
            dub(5) = s1 * t1 * (s + t + u - 1 + u) / 8
            dub(6) = s2m * t1 / 4
            dub(7) = s1m * t1 * (-s + t + u - 1 + u) / 8
            dub(8) = s1m * t2m / 4
            dub(9) = -s1m * t1m * u / 2
            dub(10) = -s1 * t1m * u / 2
            dub(11) = -s1 * t1 * u / 2
            dub(12) = -s1m * t1 * u / 2
            dub(13) = s1m * t1m * (s + t + u + 1 + u) / 8
            dub(14) = -s2m * t1m / 4
            dub(15) = s1 * t1m * (-s + t + u + 1 + u) / 8
            dub(16) = -s1 * t2m / 4
            dub(17) = s1 * t1 * (-s - t + u + 1 + u) / 8
            dub(18) = -s2m * t1 / 4
            dub(19) = s1m * t1 * (s - t + u + 1 + u) / 8
            dub(20) = -s1m * t2m / 4
        End If
    End Sub

    Sub ShellSort2(ByVal kbuf() As Integer, ByRef kcbuf() As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ｼｪﾙｿｰﾄ
        Dim i As Integer
        Dim nsort(Npoint) As Integer
        Dim work(Npoint) As Single

        Select Case kbuf(1)
            Case 1
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Xdata(i)
                Next i
            Case 2
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Ydata(i)
                Next i
            Case 3
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Zdata(i)
                Next i
        End Select

        Call ShellSortStd(Npoint, nsort, work)    'ｼｪｯﾙｿｰﾄ基本

        Call MiniSort2(kbuf, nsort, work, xbuf, ybuf, zbuf)    'ﾐﾆｿｰﾄ

        For i = 1 To Npoint
            kcbuf(nsort(i)) = i
        Next i

    End Sub

    Sub ShellSortStd(ByVal nx As Integer, ByRef nsort() As Integer, ByRef work() As Single)
        'ｼｪﾙｿｰﾄ基本
        'nx･･･変数の数
        'nsort()･･ｿｰﾄ順の元変数の番号
        'work()･･･ｿｰﾄ順の変数値
        Dim i, ib, j, m As Integer
        Dim varb As Single

        m = nx
        Do While m > 1
            m = m \ 2
            For j = m + 1 To nx
                varb = work(j)
                ib = nsort(j)
                i = j - m
                Do While work(i) >= varb
                    work(i + m) = work(i)
                    nsort(i + m) = nsort(i)
                    i = i - m
                    If i <= 0 Then
                        Exit Do
                    End If
                Loop
                work(i + m) = varb
                nsort(i + m) = ib
            Next j
        Loop

    End Sub

    Sub SurfaceCond(ByVal kb As Integer, ByRef kind As Integer, _
        ByRef kbuf() As Integer, ByRef gpos() As Double)
        '境界条件用の面設定
        'kb･･･面番号
        'kind･･･面分類（1:面1と2、2:面3と4、3:面5と6）
        'kbuf()･･･面節点の基本番号
        'gpos()･･･積分点のξηζ座標
        Dim i As Integer
        Select Case kb
            Case 1
                kind = 1
                gpos(3) = 1
                For i = 1 To 8
                    kbuf(i) = i
                Next i
            Case 2
                kind = 1
                gpos(3) = -1
                kbuf(1) = 13
                kbuf(2) = 20
                kbuf(3) = 19
                kbuf(4) = 18
                kbuf(5) = 17
                kbuf(6) = 16
                kbuf(7) = 15
                kbuf(8) = 14
            Case 3
                kind = 2
                gpos(2) = -1
                kbuf(1) = 1
                kbuf(2) = 9
                kbuf(3) = 13
                kbuf(4) = 14
                kbuf(5) = 15
                kbuf(6) = 10
                kbuf(7) = 3
                kbuf(8) = 2
            Case 4
                kind = 2
                gpos(2) = 1
                kbuf(1) = 5
                kbuf(2) = 11
                kbuf(3) = 17
                kbuf(4) = 18
                kbuf(5) = 19
                kbuf(6) = 12
                kbuf(7) = 7
                kbuf(8) = 6
            Case 5
                kind = 3
                gpos(1) = 1
                kbuf(1) = 3
                kbuf(2) = 10
                kbuf(3) = 15
                kbuf(4) = 16
                kbuf(5) = 17
                kbuf(6) = 11
                kbuf(7) = 5
                kbuf(8) = 4
            Case 6
                kind = 3
                gpos(1) = -1
                kbuf(1) = 7
                kbuf(2) = 12
                kbuf(3) = 19
                kbuf(4) = 20
                kbuf(5) = 13
                kbuf(6) = 9
                kbuf(7) = 1
                kbuf(8) = 8
        End Select

    End Sub

    Sub SurfaceDrawC(ByRef gr As Graphics, ByVal k1 As Integer, _
    ByVal k2 As Integer, ByVal kind As Integer)
        '面の作画（材料別色分け）
        'k1,k2･･･要素の開始・終了番号
        'kind･･･面外周の作画有無（0:なし、1:有り）
        Dim aa$
        Dim ie, j, jmax, k, kb, kk, kmax, nb, nmax As Integer
        Dim xb, xd, yb, yd As Single
        Dim fnt As Font = New Font("ＭＳ ゴシック", KnumSize)
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", KnumSb, FontStyle.Bold)

        If KindElem = 0 Then
            jmax = 6
            kmax = 8
            nmax = 20
        End If
        For nb = 1 To NSdraw
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint) '面の設定
            End If
            For k = 1 To kmax
                kk = KsurfPoint(k)
                xd = Xscreen(kk)
                yd = Yscreen(kk)
                Call XYCalc(xd, yd)     'X,Yの作画座標計算
                Xdraw(k) = xd
                Ydraw(k) = yd
            Next k
            kk = (NumMat(ie) - 1) Mod 7
            Select Case kk
                Case 0
                    Call PaintExe(gr, kmax, Color.White, kind)
                Case 1
                    Call PaintExe(gr, kmax, Color.LightBlue, kind)
                Case 2
                    Call PaintExe(gr, kmax, Color.LightGreen, kind)
                Case 3
                    Call PaintExe(gr, kmax, Color.LightCyan, kind)
                Case 4
                    Call PaintExe(gr, kmax, Color.LightPink, kind)
                Case 5
                    Call PaintExe(gr, kmax, Color.LightYellow, kind)
                Case 6
                    Call PaintExe(gr, kmax, Color.LightGray, kind)
            End Select
            If KnumDisp > 0 Then
                xb = 0 : yb = 0
                For k = 1 To kmax
                    xb = xb + Xdraw(k)
                    yb = yb + Ydraw(k)
                    If KdispPNum > 0 Then
                        kk = KsurfPoint(k)
                        xd = Xdraw(k)
                        yd = Ydraw(k)
                        aa$ = LTrim$(Str$(kk))
                        gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                    End If
                Next k
                If KdispSNum > 0 Then
                    xd = xb / kmax
                    yd = yb / kmax
                    aa$ = "(" + LTrim$(Str$(kb)) + ")"
                    gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                End If
                If KdispENum > 0 Then
                    xb = 0 : yb = 0
                    For j = 1 To nmax
                        kb = NNelem(ie, j)
                        xd = Xscreen(kb)
                        yd = Yscreen(kb)
                        Call XYCalc(xd, yd) 'X,Yの作画座標計算
                        xb = xb + xd
                        yb = yb + yd
                    Next j
                    xd = xb / nmax
                    yd = yb / nmax
                    aa$ = LTrim$(Str$(ie))
                    gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
                End If
            End If
        Next nb

        If k1 <= KmouseElem And KmouseElem <= k2 Then
            j = 0
            For nb = 1 To NSdraw
                ie = KEorder(nb)
                kb = KSorder(nb)
                If ie = KmouseElem And kb = KmouseSurf Then
                    j = 1
                    Exit For
                End If
            Next nb
            If j > 0 Then
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
                End If
                For k = 1 To kmax
                    kk = KsurfPoint(k)
                    xd = Xscreen(kk)
                    yd = Yscreen(kk)
                    Call XYCalc(xd, yd)     'X,Yの作画座標計算
                    Xdraw(k) = xd
                    Ydraw(k) = yd
                Next k
                xb = 0 : yb = 0
                For k = 1 To kmax
                    xb = xb + Xdraw(k)
                    yb = yb + Ydraw(k)
                    kk = KsurfPoint(k)
                    xd = Xdraw(k)
                    yd = Ydraw(k)
                    aa$ = LTrim$(Str$(kk))
                    gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                Next k
                xd = xb / kmax
                yd = yb / kmax
                aa$ = "(" + LTrim$(Str$(kb)) + ")"
                gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                xb = 0 : yb = 0
                For j = 1 To nmax
                    kb = NNelem(ie, j)
                    xd = Xscreen(kb)
                    yd = Yscreen(kb)
                    Call XYCalc(xd, yd) 'X,Yの作画座標計算
                    xb = xb + xd
                    yb = yb + yd
                Next j
                xd = xb / nmax
                yd = yb / nmax
                aa$ = LTrim$(Str$(ie))
                gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
            End If
        End If

    End Sub

    Sub SurfaceSet(ByVal ib As Integer, ByVal kb As Integer, _
       ByRef kbuf() As Integer)
        '面の設定
        'ib･･･要素番号
        'kb･･･面番号
        'kbuf()･･･面の節点番号
        Select Case kb
            Case 1
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 2)
                kbuf(3) = NNelem(ib, 3)
                kbuf(4) = NNelem(ib, 4)
                kbuf(5) = NNelem(ib, 5)
                kbuf(6) = NNelem(ib, 6)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 8)
            Case 2
                kbuf(1) = NNelem(ib, 13)
                kbuf(2) = NNelem(ib, 20)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 16)
                kbuf(7) = NNelem(ib, 15)
                kbuf(8) = NNelem(ib, 14)
            Case 3
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 9)
                kbuf(3) = NNelem(ib, 13)
                kbuf(4) = NNelem(ib, 14)
                kbuf(5) = NNelem(ib, 15)
                kbuf(6) = NNelem(ib, 10)
                kbuf(7) = NNelem(ib, 3)
                kbuf(8) = NNelem(ib, 2)
            Case 4
                kbuf(1) = NNelem(ib, 5)
                kbuf(2) = NNelem(ib, 11)
                kbuf(3) = NNelem(ib, 17)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 19)
                kbuf(6) = NNelem(ib, 12)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 6)
            Case 5
                kbuf(1) = NNelem(ib, 3)
                kbuf(2) = NNelem(ib, 10)
                kbuf(3) = NNelem(ib, 15)
                kbuf(4) = NNelem(ib, 16)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 11)
                kbuf(7) = NNelem(ib, 5)
                kbuf(8) = NNelem(ib, 4)
            Case 6
                kbuf(1) = NNelem(ib, 7)
                kbuf(2) = NNelem(ib, 12)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 20)
                kbuf(5) = NNelem(ib, 13)
                kbuf(6) = NNelem(ib, 9)
                kbuf(7) = NNelem(ib, 1)
                kbuf(8) = NNelem(ib, 8)
        End Select

    End Sub

    Sub SurfPileCheck(ByVal nb As Integer)
        '面の重なりのﾁｪｯｸ
        'nb･･･面の数
        'KAppSide(,)が1の面が重なっている場合に遠方の面は0に変更
        Dim i, ib, ic, ie, ie2, ip, j, jmax, kb, kb2, kh, nd, nmax As Integer
        Dim b1, b2, epsA, epsL As Single
        Dim ksbuf(8) As Integer
        Dim nebuf(nb), nsbuf(nb) As Integer
        Dim xbuf(8), xbuf2(8), xrange(2), xrange2(2) As Single
        Dim ybuf(8), ybuf2(8), yrange(2), yrange2(2) As Single

        If KindElem = 0 Then
            jmax = 6
            nmax = 8
        End If

        nd = 0
        For i = 1 To NSdraw
            ie = KEorder(i)
            kb = KSorder(i)
            If KAppSide(ie, kb) > 0 Then
                nd = nd + 1
                nebuf(nd) = ie
                nsbuf(nd) = kb
            End If
        Next i

        For i = 1 To nd
            ie = nebuf(i)
            If ie > 0 Then
                kb = nsbuf(i)
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)     '面の設定
                End If
                For ip = 1 To nmax
                    ib = KsurfPoint(ip)
                    xbuf(ip) = Xscreen(ib)
                    ybuf(ip) = Yscreen(ib)
                Next ip
                Call AxisCheckSet(nmax, xrange, yrange, xbuf, ybuf)
                'X･Y座標の最小・最大値のﾁｪｯｸ（ｾｯﾄ）

                b1 = xrange(2) - xrange(1)
                b2 = yrange(2) - yrange(1)
                If b2 > b1 Then
                    b1 = b2
                End If
                epsL = 0.001 * b1  '微小誤差長さ
                epsA = epsL * epsL  '微小誤差面積
                For j = i + 1 To nd
                    ie2 = nebuf(j)
                    If ie2 > 0 Then
                        kb2 = nsbuf(j)
                        If KindElem = 0 Then
                            Call SurfaceSet(ie2, kb2, ksbuf)     '面の設定
                        End If
                        For ip = 1 To nmax
                            ib = ksbuf(ip)
                            xbuf2(ip) = Xscreen(ib)
                            ybuf2(ip) = Yscreen(ib)
                        Next ip
                        Call AxisCheckSet(nmax, xrange2, yrange2, xbuf2, ybuf2)
                        'X･Y座標の最小・最大値のﾁｪｯｸ（ｾｯﾄ）

                        Call RangeCheck(kh, epsL, xrange, xrange2, yrange, yrange2)
                        '範囲ﾁｪｯｸ

                        If kh = 0 Then
                            Call AreaCheck(kh, nmax, epsA, xbuf, xbuf2, ybuf, ybuf2)
                            '2面の重なりﾁｪｯｸ

                            If kh > 0 Then
                                Call DistComp(ic, ksbuf)    '距離比較
                                If ic = 0 Then
                                    KAppSide(ie2, nsbuf(j)) = 0
                                    nebuf(j) = 0
                                Else
                                    KAppSide(ie, nsbuf(i)) = 0
                                    nebuf(i) = 0
                                    Exit For
                                End If
                            End If
                        End If
                    End If
                Next j
            End If
        Next i

    End Sub

    Sub SurfSort(ByVal k1 As Integer, ByVal k2 As Integer)
        '作画面のｿｰﾃｨﾝｸﾞ
        'k1,k2･･･開始・終了要素番号
        Dim i1, i2, ic, ie, ii, ik1, ik2, j, k, kmax, nmax, nb As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
        End If
        NSdraw = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        NSdraw = NSdraw + 1
                    End If
                Next k
            End If
        Next ie
        ReDim KEorder(NSdraw), KSorder(NSdraw)
        nb = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        nb = nb + 1
                        KEorder(nb) = ie
                        KSorder(nb) = k
                    End If
                Next k
            End If
        Next ie

        For ii = 1 To NSdraw - 1
            i1 = KEorder(ii)
            ik1 = KSorder(ii)
            If KindElem = 0 Then
                Call SurfaceSet(i1, ik1, KsurfPoint)    '面の設定
            End If

            For j = ii + 1 To NSdraw
                i2 = KEorder(j)
                ik2 = KSorder(j)
                If KindElem = 0 Then
                    Call SurfaceSet(i2, ik2, kbuf)      '面の設定
                End If

                Call DistComp(ic, kbuf)    '距離比較
                If ic = 0 Then
                    KEorder(ii) = i2
                    KSorder(ii) = ik2
                    KEorder(j) = i1
                    KSorder(j) = ik1
                    i1 = i2
                    ik1 = ik2
                    For k = 1 To nmax
                        KsurfPoint(k) = kbuf(k)
                    Next k
                End If
            Next j
        Next ii

    End Sub

    Private Sub TempComp(ByRef ic As Integer, ByVal b1 As Single, _
    ByRef tempcmax As Single, ByRef tempcmin As Single)
        '温度比較
        'ic･･･初回区分（0:初回、1:次回以降）
        'tempcmax･･･最高温度
        'tempcmin･･･最低温度
        If ic = 0 Then
            tempcmax = b1
            tempcmin = b1
            ic = 1
        ElseIf b1 < tempcmin Then
            tempcmin = b1
        ElseIf b1 > tempcmax Then
            tempcmax = b1
        End If

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\AnIsoTherm.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\AnIsoTherm.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TimCal(ByRef ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalA(ByVal xs As Single, ByVal ys As Single, _
        ByVal zs As Single, ByVal xr As Single, ByVal yr As Single, _
        ByVal zr As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        '先にZ,Y,X軸回りの回転、最後に座標軸方向の平行移動
        'xs,ys,zs･･･X,Y,Z方向移動量
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = xs
        buf1(2, 4) = ys
        buf1(3, 4) = zs

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + TransMat(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalB(ByVal xs As Single, ByVal ys As Single, ByVal zs As Single, _
    ByVal xr As Single, ByVal yr As Single, ByVal zr As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出（立体移動）
        '最初に座標軸逆方向の平行移動、次にZ,Y,X軸回りの回転
        '最後に座標軸方向の平行移動戻し
        'xs,ys,zs･･･X,Y,Z方向移動量
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = xs
        buf1(2, 4) = ys
        buf1(3, 4) = zs

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + TransMat(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                buf2(i, j) = b1
            Next j
        Next i

        For i = 1 To 4
            For j = 1 To 4
                buf1(i, j) = 0
            Next j
        Next i
        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = -xs
        buf1(2, 4) = -ys
        buf1(3, 4) = -zs
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalc(ByVal eyeh As Single, ByVal xrot As Single, _
        ByVal zrot As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        '最初に視点高さへの平行移動、次にZ,X軸回りの回転移動
        'eyeh･･･視点高さ（於ﾓﾃﾞﾙ座標）
        'xrot･･･ﾓﾃﾞﾙのX軸回りの回転角（右ねじ正、°）
        'zrot･･･ﾓﾃﾞﾙのz軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4) As Single

        b1 = Pai / 180
        thetax = xrot * b1
        thetaz = zrot * b1
        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf1(1, 1) = b1
        buf1(1, 2) = -b2
        buf1(2, 1) = b2
        buf1(2, 2) = b1
        buf1(3, 3) = 1
        buf1(4, 4) = 1
        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf2(i, k) * buf1(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For j = 1 To 4
            TransMat(3, j) = TransMat(3, j) - eyeh * TransMat(4, j)
        Next j
    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
        ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\AnIsoTUFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\AnIsoTUFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub VectorCalc(ByVal kind As Integer, ByVal numg As Integer,
        ByVal gp1 As Double, ByVal gp2 As Double, ByRef kbuf() As Integer,
        ByRef gpos() As Double,
                   ByRef xeb() As Single, ByRef yeb() As Single, ByRef zeb() As Single,
                   ByRef xgb() As Double, ByRef ygb() As Double, ByRef zgb() As Double,
                   ByRef snbuf() As Double)
        '面の法線ﾍﾞｸﾄﾙ算出
        'kind･･･面分類（1:面1と2、2:面3と4、3:面5と6）
        'numg･･･ｶﾞｳｽ点番号
        'snbuf()･･･法線ﾍﾞｸﾄﾙ
        Dim id, j, jd, jj, kgp1, kgp2, m As Integer
        Dim b1 As Double
        Dim dsb(20), dtb(20), dub(20), shapeb(20) As Double
        Dim sbuf(2, 3), svbuf(2, 3) As Double

        Select Case kind
            Case 1
                kgp1 = 1
                kgp2 = 2
                jj = 3
            Case 2
                kgp1 = 1
                kgp2 = 3
                jj = 2
            Case 3
                kgp1 = 2
                kgp2 = 3
                jj = 1
        End Select

        For j = 1 To 2
            For m = 1 To 2
                If m = 1 Then
                    Select Case j
                        Case 1
                            gpos(kgp1) = gp1 - 0.01
                        Case 2
                            gpos(kgp1) = gp1
                            gpos(kgp2) = gp2 - 0.01
                    End Select
                Else
                    Select Case j
                        Case 1
                            gpos(kgp1) = gp1 + 0.01
                        Case 2
                            gpos(kgp2) = gp2 + 0.01
                    End Select
                End If

                Call ShapeFunc2(1, gpos(1), gpos(2), gpos(3), shapeb, dsb, dtb, dub)
                '形状関数の計算

                Call JacobElem2M(numg, kbuf, shapeb, xeb, yeb, zeb, xgb, ygb, zgb, dsb, dtb, dub)
                '要素表面のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算

                sbuf(m, 1) = xgb(numg)
                sbuf(m, 2) = ygb(numg)
                sbuf(m, 3) = zgb(numg)
            Next m
            For jd = 1 To 3
                svbuf(j, jd) = sbuf(2, jd) - sbuf(1, jd)
            Next jd
        Next j
        snbuf(1) = svbuf(1, 2) * svbuf(2, 3) - svbuf(1, 3) * svbuf(2, 2)
        snbuf(2) = svbuf(1, 3) * svbuf(2, 1) - svbuf(1, 1) * svbuf(2, 3)
        snbuf(3) = svbuf(1, 1) * svbuf(2, 2) - svbuf(1, 2) * svbuf(2, 1)
        id = 0
        If kind = 2 Then
            If gpos(2) > 0 Then
                id = 1
            End If
        Else
            If gpos(jj) < 0 Then
                id = 1
            End If
        End If
        If id > 0 Then
            For m = 1 To 3
                snbuf(m) = -snbuf(m)
            Next m
        End If
        b1 = 0
        For m = 1 To 3
            b1 = b1 + snbuf(m) * snbuf(m)
        Next m
        b1 = Math.Sqrt(b1)

        For m = 1 To 3
            snbuf(m) = snbuf(m) / b1
        Next m

    End Sub

    Sub VectorCalG(ByVal ig As Integer, ByVal kg As Integer, ByRef rlen2 As Double,
    ByRef sdbuf() As Double)
        'ｶﾞｳｽ点間ﾍﾞｸﾄﾙ算出
        'ig･･･基本ｶﾞｳｽ点番号
        'kg･･･対向ｶﾞｳｽ点番号
        'rlen2･･･距離の2乗
        'sdbuf()･･･ﾍﾞｸﾄﾙ
        Dim b1, b2, b3, b4 As Double

        b1 = XG2(kg) - XG(ig)
        b2 = YG2(kg) - YG(ig)
        b3 = ZG2(kg) - ZG(ig)
        rlen2 = b1 * b1 + b2 * b2 + b3 * b3
        b4 = Math.Sqrt(rlen2)
        sdbuf(1) = b1 / b4
        sdbuf(2) = b2 / b4
        sdbuf(3) = b3 / b4

    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

End Module
